SystemOrganization addCategory: #'HedgeHacks-Painter'!
SystemOrganization addCategory: #'HedgeHacks-Painter-T3D'!

Object subclass: #T3DSimpleFace
	instanceVariableNames: 'vertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DSimpleFace class methodsFor: 'instance creation' stamp: 'ar 3/8/2006 19:21'!
with: v0 with: v1 with: v2
	^self withAll: (Array with: v0 with: v1 with: v2)! !

!T3DSimpleFace class methodsFor: 'instance creation' stamp: 'ar 3/8/2006 19:22'!
with: v0 with: v1 with: v2 with: v3
	^self withAll: (Array with: v0 with: v1 with: v2 with: v3)! !

!T3DSimpleFace class methodsFor: 'instance creation' stamp: 'ar 3/8/2006 19:21'!
withAll: aCollection
	^self new vertices: aCollection! !

!T3DSimpleFace methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:24'!
isDegenerate
	vertices size = 3 ifFalse:[^false].
	(vertices at: 1) position = (vertices at: 2) position ifTrue:[^true].
	(vertices at: 2) position = (vertices at: 3) position ifTrue:[^true].
	(vertices at: 1) position = (vertices at: 3) position ifTrue:[^true].
	^false! !

!T3DSimpleFace methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:23'!
trianglesDo: aBlock
	"Evaluate aBlock with triangular faces"
	| face |
	vertices size = 3 ifTrue:[^aBlock value: self].
	3 to: vertices size do:[:i|
		face := self class with: (vertices at: 1) with: (vertices at: i-1) with: (vertices at: i).
		face isDegenerate ifFalse:[aBlock value: face].
	].! !

!T3DSimpleFace methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:23'!
vertices
	^vertices! !

!T3DSimpleFace methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:23'!
vertices: aCollection
	vertices := aCollection! !

Object subclass: #T3DSimpleMesh
	instanceVariableNames: 'faces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DSimpleMesh class methodsFor: 'instance creation' stamp: 'ar 3/8/2006 19:15'!
withAll: aCollection
	^self new faces: aCollection! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:16'!
asIndexedMesh
	"Convert the receiver into (the more compact) indexed representation"
	^self asIndexedTriangleMesh! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:36'!
asIndexedTriangleMesh
	"Convert the receiver into (the more compact) indexed triangle representation"
	| map faceList face vtx nrm tex mesh |
	map := Dictionary new: (faces size * 4). "Need some space for the vertices"
	faceList := WriteStream on: (Array new: faces size).
	self trianglesDo:[:tri|
		face := Array
					with: (map at: tri vertices first ifAbsentPut:[map size + 1])
					with: (map at: tri vertices second ifAbsentPut:[map size + 1])
					with: (map at: tri vertices third ifAbsentPut:[map size + 1]).
		(self isDegenerateFace: face) ifFalse:[faceList nextPut: face]].
	faceList := faceList contents.
	vtx := Vector3Array new: map size.
	nrm := Vector3Array new: map size.
	tex := Vector2Array new: map size.
	map keysAndValuesDo:[:vertex :idx|
		vtx at: idx put: vertex position.
		nrm at: idx put: vertex normal.
		tex at: idx put: vertex texCoord.
	].
	mesh := T3DTriangleMesh new.
	mesh faces: faceList.
	mesh vertices: vtx.
	mesh texCoords: tex.
	mesh vertexNormals: nrm.
	^mesh! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:33'!
faces
	^faces! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:18'!
faces: aCollection
	faces := aCollection.! !

!T3DSimpleMesh methodsFor: 'testing' stamp: 'ar 3/8/2006 19:36'!
isDegenerateFace: f
	(f at: 1) = (f at: 2) ifTrue:[^true].
	(f at: 2) = (f at: 3) ifTrue:[^true].
	(f at: 3) = (f at: 1) ifTrue:[^true].
	^false! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:18'!
transformedBy: aMatrix
	"Return a copy of the receiver with its vertices transformed by the given matrix"
	| newFaces|
	newFaces := faces collect:[:face| face transformedBy: aMatrix].
	^self class withAll: newFaces! !

!T3DSimpleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:20'!
trianglesDo: aBlock
	faces do:[:f| f trianglesDo: aBlock]! !

Object subclass: #T3DSimpleVertex
	instanceVariableNames: 'position normal texCoord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DSimpleVertex methodsFor: 'comparing' stamp: 'ar 3/8/2006 23:07'!
= aVertex
	^self class == aVertex class
		and:[self position = aVertex position
			and:[self normal = aVertex normal
				and:[self texCoord = aVertex texCoord]]]! !

!T3DSimpleVertex methodsFor: 'comparing' stamp: 'ar 3/8/2006 23:06'!
hash
	"Hash is re-implemented because #= is re-implemented"
	^position hash! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:13'!
normal
	^normal! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:14'!
normal: aVector3
	normal := aVector3! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:13'!
position
	^position! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:13'!
position: aVector3
	position := aVector3! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:13'!
texCoord
	^texCoord! !

!T3DSimpleVertex methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:39'!
texCoord: uv
	texCoord := Vector2 x: uv x y: uv y! !

Object subclass: #T3DTriangle
	instanceVariableNames: 'flags e1 e2 e3 next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DTriangle commentStamp: '<historical>' prior: 0!
| faces points |
points _ (0 to: 35) collect:[:i| 
	(B3DRotation angle: i*5 axis: 0@0@1) asMatrix4x4 localPointToGlobal: (20@0@0)].
points _ points collect:[:v3| v3 x @ v3 y].
points _ {300@196. 299@205. 297@210. 296@212}.
faces _ PoohTriangle basicNew elevateFan: points to: 246@158@74 steps: 8.
faces _ B3DSimpleMesh withAll: faces.
(B3DSceneExplorerMorph new openInWorld) scene withGeometry: faces asIndexedMesh.
!

!T3DTriangle methodsFor: 'spine' stamp: 'ar 5/24/2001 12:52'!
buildTerminalFan
	| firstEdge pt points lastEdge nextEdge inside |
	self isTerminal ifFalse:[^self].
	e1 isBorderEdge ifFalse:[firstEdge := e1. pt := e3 origin].
	e2 isBorderEdge ifFalse:[firstEdge := e2. pt := e1 origin].
	e3 isBorderEdge ifFalse:[firstEdge := e3. pt := e2 origin].
	firstEdge ifNil:[^self].
	points := OrderedCollection new.
	lastEdge := firstEdge.
	points add: lastEdge destination; add: pt; add: lastEdge origin.
	nextEdge :=  lastEdge leftFace oppositeSpineEdgeOf: lastEdge symmetric.
	[lastEdge rightFace isValid: false.
	nextEdge notNil and:[nextEdge fanVertices isNil]] whileTrue:[
		nextEdge origin = points first ifTrue:[
			points addLast: nextEdge destination
		] ifFalse:[nextEdge origin = points last ifTrue:[
			points addFirst: nextEdge destination.
		] ifFalse:[nextEdge destination = points first ifTrue:[
			points addLast: nextEdge origin.
		] ifFalse:[nextEdge destination = points last ifTrue:[
			points addFirst: nextEdge origin.
		] ifFalse:[self error:'I am confused']]]].
		inside := nextEdge hasAllInsideCircle: points.
		lastEdge := nextEdge.
		inside
			ifTrue:[nextEdge := lastEdge leftFace oppositeSpineEdgeOf: lastEdge symmetric]
			ifFalse:[nextEdge := nil].
	].
	lastEdge symmetric fanVertices: points.! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 18:07'!
center
	| pt1 pt2 pt3 l1 l2 l3 sum |
	pt1 := e1 center.
	pt2 := e2 center.
	pt3 := e3 center.
	l1 := e1 length.
	l2 := e2 length.
	l3 := e3 length.
	sum := l1 + l2 + l3.
	^((pt1 * l1) + (pt2 * l2) + (pt3 * l3)) / sum
	"^(p1 + p2 + p3) / 3.0"! !

!T3DTriangle methodsFor: 'enumeration' stamp: 'ar 5/19/2001 17:29'!
do: aBlock
	| tmp |
	tmp := self.
	[tmp == nil] whileFalse:[
		aBlock value: tmp.
		tmp := tmp next].! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:45'!
e1
	^e1! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 22:08'!
e1: edge1 e2: edge2 e3: edge3
	flags := 0.
	e1 := edge1.
	e2 := edge2.
	e3 := edge3.
	e1 rightFace: self. e1 symmetric leftFace: self.
	e2 rightFace: self. e2 symmetric leftFace: self.
	e3 rightFace: self. e3 symmetric leftFace: self.
! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:45'!
e2
	^e2! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:45'!
e3
	^e3! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 3/8/2006 19:26'!
elevateConnect: vtxList1 with: vtxList2 into: aStream
	1 to: vtxList1 size-1 do:[:i|
		aStream nextPut: (T3DSimpleFace
			with: (vtxList1 at: i)
			with: (vtxList1 at: i+1)
			with: (vtxList2 at: i+1)
			with: (vtxList2 at: i))].
! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:10'!
elevateFan: vertices to: elevationVertex steps: nSteps
	"Elevate the given edge of a junction"
	| lastVtx nextVtx faces |
	faces := WriteStream on: #().
	lastVtx := nextVtx := nil.
	vertices reverseDo:[:pt|
		nextVtx := self elevateFrom: (pt@0) to: elevationVertex steps: nSteps.
		lastVtx ifNotNil:[self elevateConnect: lastVtx with: nextVtx into: faces].
		lastVtx := nextVtx.
	].
	^faces contents! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 3/8/2006 19:26'!
elevateFrom: baseVertex to: elevationVertex steps: nSteps
	| centerVertex nrm rot vtxList vtx vertex zScale dirVtx |
	centerVertex := elevationVertex x @ elevationVertex y @ 0.
	dirVtx := baseVertex - centerVertex.
	zScale := elevationVertex z / dirVtx length.
	zScale := Matrix4x4 withScale: 1@1@zScale.
	nrm := dirVtx y @ dirVtx x negated @ 0.
	vtxList := Array new: nSteps+1.
	0 to: nSteps do:[:i|
		i = 0 ifTrue:[
			vtx := baseVertex.
		] ifFalse:[i = nSteps ifTrue:[
			vtx := elevationVertex.
		] ifFalse:[
			rot := (Quaternion angle: 90.0 * i / nSteps axis: nrm) asMatrix4x4
					composedWithGlobal: zScale.
			vtx := rot localPointToGlobal: dirVtx.
			vtx := vtx + centerVertex.
		]].
		vertex := T3DSimpleVertex new.
		vertex position: vtx.
		vertex normal: Vector3 new.
		"vertex normal: (vtx - centerVertex) normalized."
		vtxList at: i+1 put: vertex].
	^vtxList! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:06'!
elevateJunction: nSteps
	| height faces |
	self isJunction ifFalse:[^#()].
	height := self elevationHeight.
	faces := WriteStream on: #().
	faces nextPutAll: (self elevateJunctionEdge: e1 height: height steps: nSteps).
	faces nextPutAll: (self elevateJunctionEdge: e2 height: height steps: nSteps).
	faces nextPutAll: (self elevateJunctionEdge: e3 height: height steps: nSteps).
	^faces contents! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:07'!
elevateJunctionEdge: anEdge height: height steps: nSteps
	"Elevate the given edge of a junction"
	| faceVertex faces edgeVertex vtxList1 vtxList2 |
	faceVertex := self center @ height.
	anEdge fanVertices ifNotNil:[
		"Connect the fan vertices with self center"
		^self elevateFan: anEdge fanVertices to: faceVertex steps: nSteps].
	"Connect 
		(anEdge origin, anEdge center, self center)
		(self center, anEdge center, anEdge destination)"
	faces := WriteStream on: #().
	edgeVertex := anEdge center @ anEdge elevationHeight.
	vtxList1 := self elevateFrom: anEdge origin @ 0 to: edgeVertex steps: nSteps.
	vtxList2 := self elevateFrom: anEdge origin @ 0 to: faceVertex steps: nSteps.
	self elevateConnect: vtxList1 with: vtxList2 into: faces.
	vtxList1 := self elevateFrom: anEdge destination @ 0 to: faceVertex steps: nSteps.
	vtxList2 := self elevateFrom: anEdge destination @ 0 to: edgeVertex steps: nSteps.
	self elevateConnect: vtxList1 with: vtxList2 into: faces.
	^faces contents! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:06'!
elevateSleeve: nSteps
	| lastEdge nextEdge faces lastEdgeVertex nextEdgeVertex vtxList1 vtxList2 |
	self isSleeve ifFalse:[^#()].
	faces := WriteStream on: #().
	e1 isBorderEdge ifTrue:[lastEdge := e2. nextEdge := e3].
	e2 isBorderEdge ifTrue:[lastEdge := e3. nextEdge := e1].
	e3 isBorderEdge ifTrue:[lastEdge := e1. nextEdge := e2].
	lastEdgeVertex := lastEdge center @ lastEdge elevationHeight.
	nextEdgeVertex := nextEdge center @ nextEdge elevationHeight.
	"Elevate fan vertices, if any"
	lastEdge fanVertices ifNotNil:[
		faces nextPutAll: (self elevateFan: lastEdge fanVertices to: lastEdgeVertex steps: nSteps)].
	nextEdge fanVertices ifNotNil:[
		faces nextPutAll: (self elevateFan: nextEdge fanVertices to: nextEdgeVertex steps: nSteps)].
	"Connect edges"
	vtxList1 := self elevateFrom: lastEdge origin @ 0 to: lastEdgeVertex steps: nSteps.
	vtxList2 := self elevateFrom: nextEdge destination @ 0to: nextEdgeVertex steps: nSteps.
	self elevateConnect: vtxList1 with: vtxList2 into: faces.
	vtxList1 := self elevateFrom: nextEdge origin @ 0 to: nextEdgeVertex steps: nSteps.
	vtxList2 := self elevateFrom: lastEdge destination @ 0 to: lastEdgeVertex steps: nSteps.
	self elevateConnect: vtxList1 with: vtxList2 into: faces.
	^faces contents! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:05'!
elevateSpine: nSteps
	"Elevate the receiver's spine"
	self isValid ifFalse:[^#()].
	self isJunction 
		ifTrue:[^self elevateJunction: nSteps].
	self isTerminal
		ifTrue:[^self elevateTerminal: nSteps].
	self isSleeve
		ifTrue:[^self elevateSleeve: nSteps].
	^#()! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/22/2001 14:06'!
elevateTerminal: nSteps
	self error:'All terminal triangles should have been removed'.! !

!T3DTriangle methodsFor: 'elevation' stamp: 'ar 5/20/2001 01:06'!
elevationHeight
	"Compute the elevation height of the receiver"
	| center sum count |
	center := self center.
	sum := (center dist: e1 origin) + (center dist: e2 origin) + (center dist: e3 origin).
	count := 3.
true ifTrue:[^sum / count].
	e1 fanVertices ifNotNilDo:[:vtx|
		vtx do:[:v| sum := sum + (v dist: center)].
		count := count + vtx size].
	e2 fanVertices ifNotNilDo:[:vtx|
		vtx do:[:v| sum := sum + (v dist: center)].
		count := count + vtx size].
	e3 fanVertices ifNotNilDo:[:vtx|
		vtx do:[:v| sum := sum + (v dist: center)].
		count := count + vtx size].
	^sum / count asFloat! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 18:25'!
flags
	^flags! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 18:25'!
flags: aNumber
	flags := aNumber! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:06'!
isJunction
	^flags anyMask: 2! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:06'!
isJunction: aBool
	flags := aBool ifTrue:[flags bitOr: 2] ifFalse:[flags bitClear: 2].! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:05'!
isSleeve
	^flags anyMask: 4! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:05'!
isSleeve: aBool
	flags := aBool ifTrue:[flags bitOr: 4] ifFalse:[flags bitClear: 4].! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:05'!
isTerminal
	^flags anyMask: 8! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:05'!
isTerminal: aBool
	flags := aBool ifTrue:[flags bitOr: 8] ifFalse:[flags bitClear: 8].! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:14'!
isValid
	^(flags bitAnd: 1) = 0! !

!T3DTriangle methodsFor: 'testing' stamp: 'ar 5/19/2001 22:14'!
isValid: aBool
	flags := aBool ifFalse:[flags bitOr: 1] ifTrue:[flags bitClear: 1].! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:27'!
next
	^next! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:28'!
next: aTriangle
	next := aTriangle.! !

!T3DTriangle methodsFor: 'spine' stamp: 'ar 5/19/2001 21:58'!
oppositeSpineEdgeOf: someEdge
	self isSleeve ifFalse:[^nil].
	someEdge == e1 ifTrue:[
		(e2 leftFace notNil and:[e3 leftFace isNil]) ifTrue:[^e2].
		(e3 leftFace notNil and:[e2 leftFace isNil]) ifTrue:[^e3].
		^nil].
	someEdge == e2 ifTrue:[
		(e1 leftFace notNil and:[e3 leftFace isNil]) ifTrue:[^e1].
		(e3 leftFace notNil and:[e1 leftFace isNil]) ifTrue:[^e3].
		^nil].
	someEdge == e3 ifTrue:[
		(e1 leftFace notNil and:[e2 leftFace isNil]) ifTrue:[^e1].
		(e2 leftFace notNil and:[e1 leftFace isNil]) ifTrue:[^e2].
		^nil].
	self error:'Edge not in receiver'! !

!T3DTriangle methodsFor: 'spine' stamp: 'ar 5/19/2001 22:03'!
recomputeType
	| count |
	count := 0.
	e1 isBorderEdge ifTrue:[count := count + 1].
	e2 isBorderEdge ifTrue:[count := count + 1].
	e3 isBorderEdge ifTrue:[count := count + 1].
	self isJunction: (count = 0).
	self isSleeve: (count = 1).
	self isTerminal: (count = 2).
! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 2/15/2006 18:31'!
v1
	^e1 origin! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 2/15/2006 18:32'!
v2
	^e2 origin! !

!T3DTriangle methodsFor: 'accessing' stamp: 'ar 2/15/2006 18:32'!
v3
	^e3 origin! !

!T3DTriangle methodsFor: 'enumeration' stamp: 'ar 5/19/2001 22:13'!
validFacesDo: aBlock
	| tmp |
	tmp := self.
	[tmp == nil] whileFalse:[
		tmp isValid ifTrue:[aBlock value: tmp].
		tmp := tmp next].! !

Object subclass: #T3DTriangleMesh
	instanceVariableNames: 'vertices vtxNormals vtxTexCoords faces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:42'!
computeFaceNormals
	| out face v1 v2 v3 d1 d2 normal |
	out := Vector3Array new: faces size.
	1 to: faces size do:[:i|
		face := faces at: i.
		v1 := vertices at: (face at: 1).
		v2 := vertices at: (face at: 2).
		v3 := vertices at: (face at: 3).
		d1 := v3 - v1.
		d2 := v2 - v1.
		d1 safelyNormalize.
		d2 safelyNormalize.
		normal := d1 cross: d2.
		out at: i put: normal safelyNormalize.
	].
	^out! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:42'!
computeVertexNormals
	| temp normals face normal v1 v2 v3 out |
	temp := Array new: vertices size.
	1 to: temp size do:[:i| temp at: i put: Vector4 new].
	normals := self computeFaceNormals.
	1 to: faces size do:[:i|
		face := faces at: i.
		normal := (normals at: i) asVector4.
		v1 := face at: 1.
		v2 := face at: 2.
		v3 := face at: 3.
		(temp at: v1) += normal.
		(temp at: v2) += normal.
		(temp at: v3) += normal.
	].
	out := Vector3Array new: vertices size.
	1 to: out size do:[:i|
		out at: i put: (temp at: i) asVector3 safelyNormalize.
	].
	^out! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:06'!
faces
	^faces! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:06'!
faces: newFaces
	faces := newFaces.! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:48'!
flipZ
	"Flip the z coordinate"
	vertices do:[:vtx| vtx z: 0.0 - vtx z].
	vtxNormals do:[:nrm| nrm z: 0.0 - nrm z].
! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:06'!
texCoords
	^vtxTexCoords! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:06'!
texCoords: newTexCoords
	vtxTexCoords := newTexCoords.
! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 20:15'!
transformBy: aMatrix
	"Modify the mesh by transforming it using a matrix; this allows us to change the insertion point of the mesh"
	vertices := vertices clone.
	vertices do:[:vtx| vtx loadFrom: (aMatrix localPointToGlobal: vtx)].
	vtxNormals := vtxNormals clone.
	vtxNormals do:[:vtx| vtx loadFrom: (aMatrix localDirToGlobal: vtx)].! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:07'!
vertexNormals
	^vtxNormals! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:07'!
vertexNormals: newNormals
	vtxNormals := newNormals.! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:08'!
vertices
	^vertices! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 19:08'!
vertices: newVertices
	vertices := newVertices.! !

!T3DTriangleMesh methodsFor: 'accessing' stamp: 'ar 3/8/2006 20:25'!
zeroBasedFaceGroup
	"Answer the faces using zero-based references."
	| myFaces index |
	myFaces := IntegerArray new: faces size * 3.
	index := 0.
	faces do:[:f|
		myFaces at: (index := index + 1) put: (f at: 1) - 1.
		myFaces at: (index := index + 1) put: (f at: 2) - 1.
		myFaces at: (index := index + 1) put: (f at: 3) - 1.
	].
	^myFaces! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 20:01'!
colorKeyAlpha
	"Form fromUser colorKeyAlpha displayOn: Display at: 0@0 rule: Form paint"
	"Use the first color in the form as a bit mask."
	| pv |
	[(BitBlt toForm: self)
		fillColor: (self colorAt: 0@0);
		combinationRule: 39;
		copyBits] on: Error do:[:ex|
		pv := bits at: 1.
		1 to: bits size do:[:i|
			(bits at: i) = pv ifTrue:[bits at: i put: 0]].
		ex return].
! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 20:01'!
colorKeyZero
	| pv |
	[(BitBlt toForm: self)
		fillColor: Color transparent;
		combinationRule: 39;
		copyBits] on: Error do:[:ex|
		pv := 0.
		1 to: bits size do:[:i|
			(bits at: i) = pv ifTrue:[bits at: i put: 0]].
		ex return].
! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 19:50'!
displayScaledOn: aForm in: aRectangle
	"Display the receiver on aForm, scaling if necessary.
		Form fromUser displayScaledOn: Display.
	"
	self extent = aForm extent ifTrue:[^self displayOn: aForm].
	(WarpBlt current toForm: aForm)
		sourceForm: self destRect: aRectangle;
		combinationRule: Form paint;
		cellSize: 2;
		warpBits.! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 20:07'!
flipVertically
	"Flip the image along the y axis"
	| rowLen row topIndex botIndex |
	self unhibernate.
	rowLen := bits size // height.
	row := Bitmap new: rowLen.
	topIndex := 1.
	botIndex := bits size - rowLen + 1.
	1 to: height // 2 do: [:i |
		self assert:[topIndex+rowLen <= botIndex].
		row replaceFrom: 1 to: rowLen with: bits startingAt: topIndex.
		bits replaceFrom: topIndex to: topIndex+rowLen-1 with: bits startingAt: botIndex.
		bits replaceFrom: botIndex to: botIndex+rowLen-1 with: row startingAt: 1.
		topIndex := topIndex + rowLen.
		botIndex := botIndex - rowLen.
	].
! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:37'!
safePixelAt: aPoint using: bitPeeker
	"Answer the color of the receiver at aPoint but test for being inside first and answer Color transparent if not"
	(aPoint x < 0 or:[aPoint y < 0]) ifTrue:[^0].
	(aPoint x >= width or:[aPoint y >= height]) ifTrue:[^0].
	^bitPeeker pixelAt: aPoint! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:28'!
smearFill: nPix
	"Fill transparent pixels in the receiver at most nPix."
	| newForm |
	newForm := Form extent: self extent depth: depth.
	1 to: nPix do:[:i|
		self displayOn: newForm at: 0@1 rule: Form paint.
		self displayOn: newForm at: 0@-1 rule: Form paint.
		self displayOn: newForm at: 1@0 rule: Form paint.
		self displayOn: newForm at: -1@0 rule: Form paint.
		self displayOn: newForm at: 0@0 rule: Form paint.
		newForm displayOn: self].
	^self! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:36'!
traceOutline: refColor
	^Array streamContents:[:s| self traceOutline: refColor do:[:pt| s nextPut: pt]].! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:37'!
traceOutline: refColor do: aBlock
	"Trace the outline edges in the receiver and evaluate aBlock with all the pixels"
	| rect x y pixelOffsets startPos lastPos pixDir nextPos refValue peeker |
	peeker := BitBlt bitPeekerFromForm: self.
	refValue := self pixelValueFor: refColor.
	pixelOffsets := {1@0. 1@1. 0@1. -1@1. -1@0. -1@-1. 0@-1. 1@-1}.
	"Find the smallest enclosing rectangle"
	rect := self rectangleEnclosingPixelsNotOfColor: refColor.
	"Find a pixel which is non-transparent"
	y := rect top. x := rect left.
	[x < rect right] whileTrue:[
		(self safePixelAt: x@y using: peeker) = refValue ifFalse:[
			startPos := x@y.
			x := rect right].
		x := x + 1.
	].
	startPos ifNil:[^self]. "this image is entirely transparent"
	nextPos := startPos.
	pixDir := 1. "we always start pointing to the right"
	[
		aBlock value: nextPos.
		lastPos := nextPos.
		"Search for a new pixel not of refColor at lastPos.
		Start with the last direction turned 90° left"
		pixDir := pixDir - 3 \\ 8 + 1.
		[nextPos := lastPos + (pixelOffsets at: pixDir).
		(self safePixelAt: nextPos using: peeker) = refValue] whileTrue:[
			"Turn left until we find something"
			pixDir := pixDir + 1 \\ 8 + 1.
		].
	nextPos = startPos] whileFalse.! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:36'!
traceOutlines
	"Answer a collection of generalized polygons generated by tracing the edges in the receiver. Each polygon's first loop is the outer boundary, all following loops are inner boundaries."
	| pts outlines outer lineSet inner mask leftOver outerOffset innerOffset |
	outer := Form extent: self extent + 2 depth: 1.
	outer offset: self offset-1.
	(BitBlt toForm: outer)
		sourceForm: self;
		colorMap: (Color maskingMap: self depth);
		destOrigin: 1@1;
		combinationRule: Form over;
		copyBits.

	outlines := OrderedCollection new.
	[pts := outer traceOutline: Color white.
	pts isEmpty] whileFalse:[
		lineSet := OrderedCollection with: pts.
		mask := self traceRegionMask: pts.
		outerOffset := mask offset.
		mask offset: 0@0.
		inner := Form extent: mask extent depth: mask depth.
		"copy outer region to inner region"
		outer displayOn: inner at: outerOffset negated rule: Form over.
		"mask out remnants"
		mask displayOn: inner at: 0@0 rule: Form and.
		"now flood-fill inner region so previous 'outside area' vanishes"
		inner floodFill: Color black at: 0@0.
		"reverse inner form so it is inside-out"
		inner reverse.

		"clear the processed area"
		mask reverse.
		mask displayOn: outer at: outerOffset rule: Form and.
		"find the interior edges"
		[pts := inner traceOutline: Color white.
		pts isEmpty] whileFalse:[
			lineSet add: pts + outerOffset.
			mask := self traceRegionMask: pts.
			innerOffset := mask offset.
			mask offset: 0@0.

			"blt the left over portion of the region back onto outer"
			leftOver := Form extent: mask extent depth: mask depth.
			inner displayOn: leftOver at: innerOffset negated rule: Form over.
			mask displayOn: leftOver at: 0@0 rule: Form and.
			leftOver floodFill: Color black at: 0@0.
			leftOver reverse.
			leftOver displayOn: outer at: innerOffset + outerOffset rule: 7. "bitOr"

			"clear the processed area"
			mask reverse.
			mask displayOn: inner at: innerOffset rule: Form and.
		].
		outlines add: lineSet.
	].
	^outlines! !

!Form methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/11/2006 14:37'!
traceRegionMask: points
	"Create a mask for the region inside points"
	| min max mask poker |
	min := points min.
	max := points max.
	"tag all the points in the outline"
	mask := Form extent: (max - min)+3 depth: 1.
	mask offset: min-1.
	poker := BitBlt bitPokerToForm: mask.
	points do:[:pt| poker pixelAt: pt - mask offset put: 1].
	"floodfill the outside area (needs border!!)"
	mask floodFill: Color black at: 0@0.
	"clean out the pixels already marked (do we need this???)"
	points do:[:pt| poker pixelAt: pt - mask offset put: 0].
	"reverse maskForm to mask out pixels not belonging to original"
	mask reverse.
	^mask! !

Model subclass: #TModel
	instanceVariableNames: 'myEventMap myScripts window'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TModel commentStamp: '<historical>' prior: 0!
TModel open.!

!TModel class methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 02:24'!
new
	^super new initialize! !

!TModel class methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 02:23'!
open
	^self new open! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/1/2004 14:10'!
camera: ac! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/12/2002 23:49'!
close
	self delete.! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/25/2002 23:22'!
closeRequest
	self close.! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/10/2002 21:15'!
delete
	window ifNotNil:[window delete].! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 02:23'!
initialize! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 03:56'!
myEventMap
	^myEventMap! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 03:56'!
myEventMap: x
	myEventMap := x! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 03:56'!
myScripts
	^myScripts ifNil:[#()]! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 03:56'!
myScripts: x
	myScripts := x.! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/25/2002 23:21'!
open
	window := TWindowMorph new.
	window model: self.

	window frame alphaSlider hResizing: #spaceFill.
	window model: self.
	window content addMorph: ((BorderedMorph new) borderColor: #inset; color: Color transparent) frame: (0@0 corner: 1@1).
	window windowColor: self windowColor.
	window extent: 256@256.
	World addMorphFront: window.
	^window! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 14:28'!
perform: aSymbol orSendTo: ignored
	^self perform: aSymbol! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/26/2002 00:47'!
postBuildIn: teapot! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 10/25/2002 22:10'!
window
	^window! !

!TModel methodsFor: 'as yet unclassified' stamp: 'ar 9/8/2002 02:47'!
windowColor
	^Color r: 0.7 g: 0.7 b: 0.4! !

TModel subclass: #TPainter
	instanceVariableNames: 'leftButton rightButton editor repaintedPlayer replaceOldCostume sketchLib'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TPainter commentStamp: '<historical>' prior: 0!
TPainter open!

!TPainter methodsFor: 'actions' stamp: 'mpm 4/20/2006 17:26'!
acceptSketch: aForm from: sketchEditor
	| rot style |

	style := #round. "#billboard #flat"
	rot := 0.
	leftButton label = 'Left' ifTrue:[rot := 0].
	leftButton label = 'Forward' ifTrue:[rot := 90].
	leftButton label = 'Right' ifTrue:[rot := 180].
	leftButton label = 'Back' ifTrue:[rot := 270].

	window owner 
		make3DObject: style
		from: aForm 
		player: repaintedPlayer
		rotateBy: rot
		replaceOldCostume: replaceOldCostume.

	self close.! !

!TPainter methodsFor: 'library' stamp: 'ar 6/17/2003 01:05'!
initiateSaveImage
	WorldState addDeferredUIMessage:[self saveImage]! !

!TPainter methodsFor: 'actions' stamp: 'ar 10/15/2002 23:01'!
leftButtonScript
	| labels newLabel |
	labels := #('Left' 'Forward' 'Right' 'Back').
	newLabel :=  labels atWrap: (labels indexOf: leftButton label) + 1.
	leftButton label: newLabel.
	newLabel :=  labels atWrap: (labels indexOf: newLabel) + 2.
	rightButton label: newLabel.
	leftButton align:  leftButton fullBounds bottomLeft with: editor bounds bottomLeft.
	rightButton align: rightButton fullBounds bottomRight with: editor bounds bottomRight.
! !

!TPainter methodsFor: 'library' stamp: 'ar 6/17/2003 01:01'!
libraryDirectory
	^FileDirectory default directoryNamed: 'Sketches'! !

!TPainter methodsFor: 'initialize' stamp: 'ar 10/15/2002 18:51'!
makeButton: label
	| button |
	button := TButtonMorph new.
	button label: label.
	button fullBounds.
	button vResizing: #rigid.
	^button! !

!TPainter methodsFor: 'initialize' stamp: 'ar 10/15/2002 22:38'!
makeButtonRow: ww
	| buttonRow cc button |
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	buttonRow := Morph new.
	buttonRow borderStyle: (BorderStyle complexAltInset width: 2).
	buttonRow borderStyle baseColor: cc.
	buttonRow color: Color transparent. "(Color white alpha: 0.5)."
	buttonRow layoutPolicy: TableLayout new.
	buttonRow listDirection: #leftToRight.
	buttonRow height: 25.

	leftButton := button := self makeButton: 'Left'.
	self runScript: #leftButtonScript when:{leftButton. #fire}.
	button addMorphFront: (LineMorph arrowFrom: 20@0 to: 0@0 width: 2).
	button hResizing: #rigid; width: 100.
	buttonRow addMorphBack: button.

	button := self makeButton: '... More stuff...'.
	button hResizing: #spaceFill.
	buttonRow addMorphBack: button.

	rightButton := button := self makeButton: 'Right'.
	self runScript: #rightButtonScript when:{rightButton. #fire}.
	button addMorphBack: (LineMorph arrowFrom: 0@0 to: 20@0 width: 2).
	button hResizing: #rigid; width: 100.
	buttonRow addMorphBack: button.

	button := self makeButton: 'Expert'.
	button hResizing: #rigid; width: ww.
	buttonRow addMorphBack: button.
	^buttonRow! !

!TPainter methodsFor: 'initialize' stamp: 'ar 6/17/2003 00:36'!
makeControlButton: aLabel after: prevButton
	| button label |
	button := window frame buildWindowButton: nil.
	label := StringMorph contents: aLabel.
	label emphasis: 1.
	aLabel size > 1 ifTrue:[button width: label width + 10].
	button addMorphCentered: label.
	window frame addMorph: button after: prevButton.
	button color: window frame closeButton color.
	^button! !

!TPainter methodsFor: 'initialize' stamp: 'ar 1/13/2006 16:02'!
makeControlButtons
	| button |
	window frame collapseButton delete.
	"window frame addMorphBack: window frame closeButton."
	button := self makeControlButton: '?' after: window frame closeButton.
	self runScript: #makeLibrary when:{button. #fire}.
	button := self makeControlButton: 'save' after: button.
	self runScript: #initiateSaveImage when:{button. #fire}.

	"button := self makeControlButton: '<' after: window frame alphaSlider.
	self runScript: #gotoPrevCostume when:{button. #fire}.
	button := self makeControlButton: '+' after: button.
	self runScript: #insertCostume when:{button. #fire}.
	button := self makeControlButton: '>' after: button.
	self runScript: #gotoNextCostume when:{button. #fire}."! !

!TPainter methodsFor: 'initialize' stamp: 'ar 10/15/2002 23:02'!
makeInplaceArrows
	| cc |
	cc := Color black.
	leftButton := self makeButton: 'Left'.
	leftButton beBordered.
	leftButton labelColor: cc; highlightColor: Color black.
	leftButton listDirection: #topToBottom; cellPositioning: #topLeft.
	self runScript: #leftButtonScript when:{leftButton. #fire}.
	leftButton borderWidth: 0; color: Color transparent.
	leftButton addMorphFront: (LineMorph arrowFrom: 20@0 to: 0@0 width: 2 color: cc).

	rightButton := self makeButton: 'Right'.
	rightButton beBordered.
	rightButton labelColor: cc; highlightColor: Color black.
	rightButton listDirection: #topToBottom; cellPositioning: #topRight.
	rightButton borderWidth: 0; color: Color transparent.
	self runScript: #rightButtonScript when:{rightButton. #fire}.
	rightButton addMorphFront: (LineMorph arrowFrom: 0@0 to: 20@0 width: 2 color: cc).! !

!TPainter methodsFor: 'library' stamp: 'ar 1/13/2006 16:04'!
makeLibrary
	| sketchList cc top scroller button |
	sketchLib ifNotNil:[
		sketchLib delete.
		sketchLib := nil.
		window extent: window content submorphBounds extent.
		^self].
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	top := sketchLib := Morph new.
	top color: (Color white mixed: 0.4 with: cc).
	top borderStyle: (BorderStyle complexAltInset width: 3).
	top borderStyle baseColor: cc.
	top extent: window content width @ 84.
	top layoutPolicy: TableLayout new.
	top listDirection: #leftToRight.
	top hResizing: #spaceFill; vResizing: #shrinkWrap.
	window addMorphBack: top.

	scroller := TransformMorph new.
	scroller color: Color transparent.
	scroller height: 84.
	scroller width: (window content innerBounds width truncateTo: 68) + 2.
	"scroller hResizing: #spaceFill."
	scroller clipSubmorphs: true.
	top addMorphBack: scroller.

	button := self makeButton:'<'. button layoutInset: 0.
	button wrapCentering: #center.
	button fillStyle direction: 0@80.
	button hResizing: #spaceFill; vResizing: #spaceFill.
	self startScript: #scrollBack: withArguments:{scroller} when:{button. #fire}.
	top addMorphFront: button.

	button := self makeButton:'>'. button layoutInset: 0.
	button hResizing: #spaceFill; vResizing: #spaceFill.
	button wrapCentering: #center.
	button fillStyle direction: 0@80.
	self startScript: #scrollForward: withArguments:{scroller} when:{button. #fire}.
	top addMorphBack: button.

	sketchList := Morph new color: Color transparent.
	sketchList layoutPolicy: TableLayout new.
	sketchList listDirection: #leftToRight.
	sketchList hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	sketchList layoutInset: 2@4; cellInset: 2.
	scroller addMorphBack: sketchList.
"
	self waitTick.
	self startScript: #updateLibrary: withArguments:{sketchList}.
"
	Cursor wait showWhile:[self updateLibrary: sketchList].! !

!TPainter methodsFor: 'library' stamp: 'ar 6/17/2003 00:43'!
makeLibraryButtonFrom: file
	| form holder cc sketch label colorKey |
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	form := Form fromBinaryStream: file.
	colorKey := form colorAt: 0@0.
	colorKey = Color transparent ifFalse:[
		form replaceColor: colorKey withColor: Color transparent.
	].
	holder := Morph new color: (Color white mixed: 0.7 with: cc).
	holder extent: 66@76.
	holder borderStyle: (BorderStyle width: 1 color: Color black).
	holder clipSubmorphs: true.
	holder cornerStyle: #rounded.
	sketch := SketchMorph withForm: form.
	sketch extent: sketch extent * (64.0 / (form width max: form height)).
	sketch position: 64 - sketch extent // 2 + 1.
	holder addMorphBack: sketch.
	label := StringMorph contents: (file localName allButLast: 4).
	label width < sketch width ifTrue:[
		label align: label bounds bottomCenter with: holder innerBounds bottomCenter.
	] ifFalse:[
		label align: label bounds bottomLeft with: holder innerBounds bottomLeft.
	].
	holder addMorphBack: label.
	holder on: #mouseDown send: #mouseDownSketch:from: to: self.
	holder on: #mouseUp send: #mouseUpSketch:from: to: self.
	^holder! !

!TPainter methodsFor: 'initialize' stamp: 'ar 10/15/2002 23:01'!
makePainter
	| box cc paintArea content boxHolder buttonRow |
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	content := window content.

	boxHolder := Morph new color: (Color white mixed: 0.4 with: cc).
	box := PaintBoxMorph new.
	boxHolder extent: box width @ content innerBounds height.
	boxHolder addMorph: box.
	box position: boxHolder extent - box fullBounds extent // 2.
	boxHolder position: content innerBounds topRight - (box width @ 0).
	content addMorph: boxHolder.

	paintArea := Morph new.
	paintArea position: content position.
	paintArea extent: content extent - (box width@0).
	paintArea borderStyle: (BorderStyle complexAltInset width: 3).
	paintArea borderStyle baseColor: cc.
	paintArea color: Color transparent. "(Color white alpha: 0.5)."
	content addMorphBack: paintArea.

	editor := TSketchEditorMorph new.
	editor teapot: self.
	editor color: Color transparent.
	editor bounds: paintArea innerBounds.
	box focusMorph: editor.
	paintArea addMorphFront: editor.
false ifTrue:[
	buttonRow := self makeButtonRow: boxHolder width.
	buttonRow width: content innerBounds width.
	buttonRow position: paintArea bottomLeft.
	content addMorphBack: buttonRow.
] ifFalse:[
	self makeInplaceArrows.
	editor addMorph: leftButton.
	editor addMorph: rightButton.
	leftButton align:  leftButton fullBounds bottomLeft with: editor bounds bottomLeft.
	rightButton align: rightButton fullBounds bottomRight with: editor bounds bottomRight.
].! !

!TPainter methodsFor: 'library' stamp: 'ar 6/16/2003 22:51'!
mouseDownSketch: evt from: holder
	| cc |
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	holder color: cc.! !

!TPainter methodsFor: 'library' stamp: 'ar 6/16/2003 22:52'!
mouseUpSketch: evt from: holder
	| cc |
	cc := (Color r: 0.7 g: 0.7 b: 0.4).
	holder color: (Color white mixed: 0.7 with: cc).
	editor repaint: holder firstSubmorph form.! !

!TPainter methodsFor: 'initialize' stamp: 'ar 6/16/2003 23:43'!
open
	window := TWindowMorph new.
	window model: self.

	window frame expandButton delete.
	window frame alphaSlider hResizing: #spaceFill.
	window model: self.
	window windowColor: self windowColor.
	window label: 'Tea Paint'.
	window extent: 440@350.
	window fullBounds.

	self makePainter.
	self makeControlButtons.

	World addMorphFront: window.
	^window! !

!TPainter methodsFor: 'actions' stamp: 'jh 5/19/2003 23:17'!
repaint: aPlayer replaceOldCostume: aBoolean
	editor repaint: aPlayer sketch.
	repaintedPlayer := aPlayer.
	replaceOldCostume := aBoolean.! !

!TPainter methodsFor: 'actions' stamp: 'ar 10/15/2002 23:01'!
rightButtonScript
	| labels newLabel |
	labels := #('Left' 'Forward' 'Right' 'Back').
	newLabel :=  labels atWrap: (labels indexOf: rightButton label) + 1.
	rightButton label: newLabel.
	newLabel :=  labels atWrap: (labels indexOf: newLabel) + 2.
	leftButton label: newLabel.
	leftButton align:  leftButton fullBounds bottomLeft with: editor bounds bottomLeft.
	rightButton align: rightButton fullBounds bottomRight with: editor bounds bottomRight.
! !

!TPainter methodsFor: 'library' stamp: 'ar 6/17/2003 00:55'!
saveImage
	"Save this guy"
	| type aForm fileName file writer |
	aForm := editor paintingForm ifNil:[^self].
	aForm colorCount <= 256 ifTrue:[type := '.gif'] ifFalse:[type := '.bmp'].
	fileName := 'Sketch'.
	[fileName := FillInTheBlankMorph request: 'Please enter the name' initialAnswer: fileName centerAt: window bounds center.
	fileName isEmptyOrNil ifTrue:[^self].
	[file := self libraryDirectory newFileNamed: fileName, type] 
		on: FileExistsException do:[:ex| ex return: nil].
	file == nil] whileTrue:[self inform: 'Error: ', fileName,' already exists'].

	[aForm := editor paintingForm.
	aForm colorAt: 0@0 put: Color transparent. "color key"
	type = '.gif' 
		ifTrue:[writer := GIFReadWriter]
		ifFalse:[writer := BMPReadWriter].
	writer putForm: aForm onStream: file.
	] ensure: file close.! !

!TPainter methodsFor: 'library' stamp: 'ar 6/16/2003 23:24'!
scrollBack: aScroller
	aScroller hasSubmorphs ifFalse:[^self].
	aScroller offset: ((aScroller offset x - 68 max: 0) truncateTo: 68) @ 0.! !

!TPainter methodsFor: 'library' stamp: 'ar 6/16/2003 23:21'!
scrollForward: aScroller
	aScroller hasSubmorphs ifFalse:[^self].
	aScroller offset: (aScroller offset x + 68 min: aScroller submorphBounds width) @ 0.! !

!TPainter methodsFor: 'library' stamp: 'ar 1/13/2006 16:03'!
updateLibrary: sketchList
	| dir extensions files file holder |
	dir := self libraryDirectory.
	extensions := #('.gif' '.bmp' '.jpg' '.png').
	files := dir fileNames select:[:fName|
				extensions anySatisfy:[:ext| fName endsWith: ext].
	].
	files := files sort.
	files do:[:fName|
		file := dir readOnlyFileNamed: fName.
		[holder := self makeLibraryButtonFrom: file] ensure:[file close].
		sketchList addMorphBack: holder.
		"self waitTick."
	].! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/6/2006 22:42'!
alpha
	^self color alpha! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/6/2006 22:42'!
alpha: aNumber
	self color: (self color alpha: aNumber)! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 18:11'!
myEventMap
	^self valueOfProperty: #myEventMap! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 18:11'!
myEventMap: aMap
	^self setProperty: #myEventMap toValue: aMap! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 18:11'!
myScripts
	^self valueOfProperty: #myScripts ifAbsent:[#()]! !

!Morph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/7/2006 18:11'!
myScripts: scriptList
	"Add aScript to the receiver's list of scripts (if any)"
	self setProperty: #myScripts toValue: scriptList! !

Morph subclass: #TButtonMorph
	instanceVariableNames: 'label highlightColor oldColor oldWidth highlightWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TButtonMorph commentStamp: '<historical>' prior: 0!
TButtonMorph new openInWorld!

!TButtonMorph methodsFor: 'initialize' stamp: 'ar 10/15/2002 22:55'!
beBordered
	| newLabel |
	newLabel := BorderedStringMorph contents: self label.
	self replaceSubmorph: label by: newLabel.
	label := newLabel.! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:30'!
contents
	^self label! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:30'!
contents: aString
	self label: aString.! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/15/2002 18:16'!
handlesMouseDown: evt
	^true! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/15/2002 18:18'!
handlesMouseOver: evt
	^true! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/15/2002 18:19'!
handlesMouseOverDragging: evt
	^true! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 22:44'!
highlightColor
	^highlightColor! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 22:44'!
highlightColor: aColor
	highlightColor := aColor.
! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/25/2002 20:43'!
highlightWidth
	^highlightWidth! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/25/2002 20:43'!
highlightWidth: aNumber
	highlightWidth := aNumber! !

!TButtonMorph methodsFor: 'initialize' stamp: 'ar 10/25/2002 20:24'!
initialize
	super initialize.
	highlightColor := Color white.
	label := StringMorph new contents: 'Button'.
	label emphasis: 1.
	self addMorphBack: label.
	self layoutPolicy: TableLayout new.
	self listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	self listCentering: #center.
	self layoutInset: 10@2.
	self cellInset: 10@0.
	self fillStyle: (GradientFillStyle colors: {
		Color r: 0.7 g: 0.7 b: 0.4.
		Color r: 1 g: 1 b: 0.95.
	}).
	self fillStyle origin: 0@0.
	self fillStyle direction: 0@18.
	self borderStyle: (BorderStyle complexRaised width: 1).
! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 18:20'!
label
	^label contents! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 18:20'!
label: aString
	^label contents: aString! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 22:47'!
labelColor
	^label color! !

!TButtonMorph methodsFor: 'accessing' stamp: 'ar 10/15/2002 22:46'!
labelColor: aColor
	label color: aColor! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/15/2002 18:17'!
mouseDown: evt
	self borderStyle: (BorderStyle complexInset width: self borderWidth)! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/25/2002 20:45'!
mouseEnter: evt
	oldColor := label color.
	label color: highlightColor.
	highlightWidth ifNotNil:[
		oldWidth := self borderWidth.
		self borderWidth: highlightWidth.
	].! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/25/2002 20:25'!
mouseEnterDragging: evt
	oldColor := label color.
	label color: highlightColor.! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 6/16/2003 22:54'!
mouseLeave: evt
	label color: oldColor.
	oldWidth ifNotNil:[self borderWidth: oldWidth].! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/25/2002 20:25'!
mouseLeaveDragging: evt
	label color: oldColor.! !

!TButtonMorph methodsFor: 'event handling' stamp: 'ar 10/25/2002 20:45'!
mouseUp: evt
	self borderStyle: (BorderStyle complexRaised width: self borderWidth).
	(self containsPoint: evt position) ifTrue:[self signal: #fire].! !

Morph subclass: #TWindowFrameMorph
	instanceVariableNames: 'closeButton expandButton collapseButton doItButton label alphaSlider'
	classVariableNames: 'CloseButtonForm ExpandButtonForm'
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TWindowFrameMorph class methodsFor: 'instance creation' stamp: 'ar 9/6/2002 21:53'!
initialize
	"TWindowFrameMorph initialize"
	| proto |
	proto := (ScriptingSystem formAtKey: 'expandBox').
	proto ifNotNil:[
		ExpandButtonForm := proto rotateBy: #left centerAt: 0@0.
	].
	proto := ScriptingSystem formAtKey:'Halo-Dismiss'.
	proto ifNotNil:[
		CloseButtonForm := Form extent: proto extent depth: 8.
		(BitBlt toForm: CloseButtonForm) sourceForm: proto; combinationRule: 3; copyBits.
	].! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:44'!
alphaSlider
	^alphaSlider! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:22'!
buildAlphaSlider
	| slider |
	slider := SimpleSliderMorph new.
	slider extent: 100@10.
	slider target: self.
	slider actionSelector: #alpha:.
	^slider! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:51'!
buildCloseButton
	^self buildWindowButton: CloseButtonForm! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 10/26/2002 19:50'!
buildDoItButton
	| button doIt |
	button := (BorderedMorph new)
		extent: 16@16;
		cornerStyle: #rounded;
		color: Color gray;
		borderStyle: (BorderStyle raised width: 2);
		yourself.
	button on: #mouseDown send: #mouseDown:from: to: self.
	button on: #mouseUp send: #mouseUp:from: to: self.
	doIt := StringMorph contents: '!!'.
	doIt emphasis: 1.
	button addMorphCentered: doIt.
	doItButton := button.
	^doItButton! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:51'!
buildExpandButton
	^self buildWindowButton: ExpandButtonForm! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:50'!
buildWindowButton
	^self buildWindowButton: nil! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:55'!
buildWindowButton: graphic
	| button icon |
	button := (BorderedMorph new)
		extent: 16@16;
		cornerStyle: #rounded;
		color: Color gray;
		borderStyle: (BorderStyle raised width: 2);
		yourself.
	button on: #mouseDown send: #mouseDown:from: to: self.
	button on: #mouseUp send: #mouseUp:from: to: self.
	graphic ifNil:[^button].
	icon := SketchMorph withForm: graphic.
	button addMorphCentered: icon.
	^button! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 9/6/2002 21:22'!
buildWindowLabel
	^(StringMorph contents: 'Tea Window') color: Color white; emphasis: 1! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:30'!
closeButton
	^closeButton! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 10/25/2002 21:00'!
collapse
	alphaSlider visible: false.! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:30'!
collapseButton
	^collapseButton! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:52'!
doItButton
	^doItButton! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 10/25/2002 21:00'!
expand
	alphaSlider visible: true.! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:30'!
expandButton
	^expandButton! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 10/26/2002 19:42'!
initialize
	| spacer |
	super initialize.
	self layoutPolicy: TableLayout new.
	self listDirection: #leftToRight.
	self cellPositioning: #leftCenter.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.
	self layoutInset: 4@4; cellInset: 4@0.
	self addMorphBack: (closeButton := self buildCloseButton).
	label := self buildWindowLabel.
	spacer := (Morph new) color: Color transparent; hResizing: #spaceFill; vResizing: #spaceFill.
	spacer clipSubmorphs: true.
	spacer addMorphBack: label.
	self addMorphBack: spacer.
	self addMorphBack: (alphaSlider := self buildAlphaSlider).
	self addMorphBack: (collapseButton := self buildWindowButton).
	self addMorphBack: (expandButton := self buildExpandButton).
! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:03'!
label
	^label contents! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:03'!
label: aString
	label contents: aString! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:31'!
labelColor
	^label labelColor! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:31'!
labelColor: aColor
	label labelColor: aColor! !

!TWindowFrameMorph methodsFor: 'initialize' stamp: 'ar 10/26/2002 19:43'!
makeLabelButton
	| newLabel |
	newLabel := TButtonMorph new.
	newLabel fillStyle direction: newLabel fillStyle direction * 2.
	"newLabel cornerStyle: #rounded."
	newLabel label: self label.
	newLabel labelColor: self labelColor.
	newLabel fullBounds.
	newLabel position: label position.
	newLabel highlightWidth: 2.
	label owner replaceSubmorph: label by: newLabel.
	label delete.
	label := newLabel.
	^label! !

!TWindowFrameMorph methodsFor: 'events' stamp: 'ar 9/6/2002 21:55'!
mouseDown: evt from: aButton
	aButton borderColor: #inset.! !

!TWindowFrameMorph methodsFor: 'events' stamp: 'ar 9/6/2002 22:43'!
mouseUp: evt from: aButton
	aButton borderColor: #raised.
	(aButton containsPoint: evt position) ifTrue:[aButton signal: #fire].! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:21'!
window: aTWindow
	alphaSlider target: aTWindow.
	alphaSlider value: aTWindow alpha.
! !

!TWindowFrameMorph methodsFor: 'accessing' stamp: 'ar 10/26/2002 19:50'!
windowColor: windowColor
	| buttonColor |
	self fillStyle: (GradientFillStyle colors: {windowColor. Color white mixed: 0.65 with: windowColor}).
	self fillStyle origin: 0@0; direction: 0@24.
	buttonColor := (windowColor mixed: 0.3 with: Color white).
	closeButton color: buttonColor.
	collapseButton color: buttonColor.
	expandButton color: buttonColor.
	doItButton ifNotNil:[doItButton color: buttonColor].! !

!LineMorph class methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/6/2006 22:43'!
arrowFrom: startPoint to: endPoint width: lineWidth color: aColor

	^ (PolygonMorph vertices: {startPoint. endPoint}
			color: aColor borderWidth: lineWidth borderColor: aColor) makeForwardArrow! !

Morph subclass: #TSketchEditorMorph
	instanceVariableNames: 'palette paintingForm formCanvas undoBuffer teapot'
	classVariableNames: ''
	poolDictionaries: 'OpenGLConstants'
	category: 'HedgeHacks-Painter'!

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 6/16/2003 23:37'!
accept
	| newForm newBox |
	teapot ifNil:[^self].
	newBox := Rectangle origin: (0@0) extent: (paintingForm extent).
	newForm := 	Form extent: newBox extent depth: paintingForm depth.
	newForm copyBits: newBox from: paintingForm at: 0@0
		clippingBox: newForm boundingBox rule: Form over fillColor: nil.
	newForm isAllWhite ifTrue: [^self cancel].
	teapot acceptSketch: newForm from: self.! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:09'!
action
	^self valueOfProperty: #action! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:09'!
action: anAction
	^self setProperty: #action toValue: anAction! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:10'!
brush
	^self valueOfProperty: #brush! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:10'!
brush: anObject
	^self setProperty: #brush toValue: anObject! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 10/12/2002 23:48'!
cancel
	teapot ifNil:[^self].
	^teapot close.! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 13:35'!
cancelPainting: aPaintBoxMorph evt: evt
	"Undo the operation after user issued #cancel in aPaintBoxMorph"
	self cancel.! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 8/20/2002 02:24'!
clear
	"wipe out all the paint"

	paintingForm fillWithColor: Color transparent.
	self invalidRect: bounds.! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 02:05'!
clearPainting: aPaintBoxMorph
	"Clear the image after user issued #clear in aPaintBoxMorph"
	^self clear! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:01'!
drawOn: aCanvas
	"Put the painting on the display"

	color isTransparent ifFalse: [
		aCanvas fillRectangle: bounds color: color
	].
	paintingForm ifNotNil: [
		aCanvas paintImage: paintingForm at: bounds origin].

 ! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 8/20/2002 02:03'!
erase: evt
	"Pen is set up to draw transparent squares"
	self paint: evt
! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 8/20/2002 02:17'!
erasePrep: evt
	"Transparent paint, square brush.  Be careful not to let this be undone by asking palette for brush and color."

	| size pfPen myBrush |

	pfPen := self paintingFormPen.
	size := (palette getNib) width.
	self brush: (myBrush := Form extent: size@size depth: 1).
	myBrush offset: (0@0) - (myBrush extent // 2).
	myBrush fillWithColor: Color black.
	pfPen sourceForm: myBrush.
	"transparent"
	pfPen combinationRule: Form erase1bitShape.
	pfPen color: Color black.
! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 6/19/2003 23:21'!
fill: evt 
	"Find the area that is the same color as where you clicked. Fill it with 
	the current paint color."
	| box |
	evt isMouseUp
		ifFalse: [^ self].
	palette recentColor: palette getColor.
	"Only fill upon mouseUp"
	"would like to only invalidate the area changed, but can't find out what it is."
	Cursor execute
		showWhile: [
			box := paintingForm
				floodFill: palette getColor
				at: evt cursorPoint - bounds origin.
			self invalidRect: (box translateBy: bounds origin)]! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:01'!
handlesMouseDown: evt

	^ true
! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:02'!
handlesMouseOver: evt
	^true! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:02'!
handlesMouseOverDragging: evt
	^true! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:10'!
lastEvent
	^self valueOfProperty: #lastEvent! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:09'!
lastEvent: anObject
	^self setProperty: #lastEvent toValue: anObject! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:20'!
mouseDown: evt
	"Start a new stroke.  Check if any palette setting have changed.  6/11/97 20:30 tk"
	| cur pfPen myAction |
	self prepareToPaint: evt.
	"verify that we are in a good state"
	pfPen := self paintingFormPen.
	undoBuffer := paintingForm deepCopy.	"know we will draw something"
	pfPen place: (evt cursorPoint - bounds origin).
	myAction := palette action.
	myAction == #paint: ifTrue:[
		palette recentColor: palette getColor].
		"origin point for pickup: rect: ellispe: polygon: line: star:.  Always take it."
	myAction == #pickup: ifTrue: [
		cur := Cursor corner clone.
		cur offset: 0@0  "cur offset abs".
		evt hand showTemporaryCursor: cur].
	myAction == #polygon: ifTrue: [self polyNew: evt].	"a mode lets you drag vertices"
	self mouseMove: evt.! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:40'!
mouseEnter: evt
	"Set the cursor.  Reread colors if embedded editable polygon needs it."

	| poly cColor |
	super mouseEnter: evt.
	"evt hand showTemporaryCursor: palette plainCursor."
	palette getSpecial == #polygon: ifFalse: [^self].
	(poly := self valueOfProperty: #polygon) ifNil: [^ self].
	cColor := palette getColor.
	poly color: cColor; borderWidth: 1.
	poly changed.! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:01'!
mouseEnterDragging: evt
	"Test button state elsewhere if at all"
	^ self mouseEnter: evt! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:01'!
mouseLeave: evt
	"Revert to the normal hand cursor."

	super mouseLeave: evt.
	evt hand showTemporaryCursor: nil.  "back to normal"
	"If this is modified to close down the SketchEditorMorph in any way, watch out for how it is called when entering a rotationButton and a scaleButton."
! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:02'!
mouseLeaveDragging: evt
	"Test button state elsewhere if at all"
	^ self mouseLeave: evt! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:15'!
mouseMove: evt
	"In the middle of drawing a stroke.  6/11/97 19:51 tk"

	| pt priorEvt |

	WorldState canSurrenderToOS: false.		"we want maximum responsiveness"
	pt := evt cursorPoint.
	priorEvt := self lastEvent.
	(priorEvt ~~ nil and: [pt = priorEvt cursorPoint]) ifTrue: [^ self].
	self perform: (palette action) with: evt.
		"Each action must do invalidRect:"
	self lastEvent: evt.
	false ifTrue: ["So senders will find the things performed here"
		self paint: nil; fill: nil; erase: nil; pickup: nil; stamp: nil.
		self rect: nil; ellipse: nil; polygon: nil; line: nil; star: nil].! !

!TSketchEditorMorph methodsFor: 'drawing' stamp: 'ar 8/20/2002 02:18'!
mouseUp: evt
	| myAction |
	"Do nothing except those that work on mouseUp."

	myAction := palette action.
	myAction == #fill: ifTrue: [
		self perform: myAction with: evt.
		"Each action must do invalidRect:"
		].
	myAction == #pickup: ifTrue: [
		self pickupMouseUp: evt].
	myAction == #polygon: ifTrue: [self polyEdit: evt].	"a mode lets you drag vertices"
	self lastEvent: nil.
! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 9/19/2002 15:59'!
paint: evt
	"While the mouse is down, lay down paint, but only within window bounds.
	 11/28/96 sw: no longer stop painting when pen strays out of window; once it comes back in, resume painting rather than waiting for a mouse up"

	|  pfPen myBrush lastPoint dirtyRect trail nextPoint |
	pfPen := self paintingFormPen.
	myBrush := palette getNib.
	lastPoint := pfPen location.
	dirtyRect := lastPoint + myBrush offset extent: myBrush extent.
	evt isMouseMove 
		ifTrue:[trail := evt trail copyWith: evt position]
		ifFalse:[trail := Array with: evt position].
	1 to: trail size do:[:i|
		nextPoint := (trail at: i) - bounds origin.
		pfPen goto: nextPoint.
		dirtyRect := dirtyRect merge: (nextPoint + myBrush offset extent: myBrush extent).
		lastPoint := nextPoint.
	].
	self invalidRect: (dirtyRect translateBy: bounds origin).! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 02:31'!
paintBoxChanged: arguments
! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 6/17/2003 00:51'!
paintingForm
	| newForm newBox |
	paintingForm ifNil:[^nil].
	newBox := Rectangle origin: (0@0) extent: (paintingForm extent).
	newForm := 	Form extent: newBox extent depth: paintingForm depth.
	newForm copyBits: newBox from: paintingForm at: 0@0
		clippingBox: newForm boundingBox rule: Form over fillColor: nil.
	newForm isAllWhite ifTrue: [^nil].
	^newForm! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:08'!
paintingFormPen
	^self valueOfProperty: #paintingFormPen! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:08'!
paintingFormPen: aPen
	self setProperty: #paintingFormPen toValue: aPen! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 02:02'!
palette
	^palette! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 02:14'!
paletteAttached: aPaintBoxMorph
	"A new palette has been attached to the receiver."
	palette := aPaintBoxMorph.! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 02:06'!
paletteDetached: aPaintBoxMorph
	"The palette has been detached to the receiver.
	Don't know what to do here...."! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 8/20/2002 02:22'!
prepareToPaint: evt
	"Figure out what the current brush, fill, etc is.  Return an action to take every mouseMove.  Set up instance variable and pens.  Prep for normal painting is inlined here.  tk 6/14/97 21:11"

	| specialMode pfPen cColor cNib myBrush |
	paintingForm ifNil:[
		paintingForm := Form extent: bounds extent depth: 32.
	].
	"Install the brush, color, (replace mode), and cursor."
	specialMode := palette action.
 	cColor  := palette getColor.
	cNib := palette getNib.
	self brush: (myBrush := cNib).
	self paintingFormPen: (pfPen := Pen newOnForm: paintingForm).
	formCanvas := paintingForm getCanvas.	"remember to change when undo"
	formCanvas := formCanvas
		copyOrigin: self topLeft negated
		clipRect: (0@0 extent: bounds extent).

	specialMode == #paint: ifTrue: [
		"get it to one bit depth.  For speed, instead of going through a colorMap every time ."
		self brush: (myBrush := Form extent: myBrush extent depth: 1).
		myBrush offset: (0@0) - (myBrush extent // 2).
		cNib displayOn: myBrush at: (0@0 - cNib offset).

		pfPen sourceForm: myBrush.
		pfPen combinationRule: Form paint.
		pfPen color: cColor.
		cColor isTransparent ifTrue: [
			pfPen combinationRule: Form erase1bitShape.
			pfPen color: Color black].
		^ #paint:].

	specialMode == #erase: ifTrue: [
		self erasePrep: evt.
		^ #erase:].

	(self respondsTo: specialMode) 
		ifTrue: [^ specialMode]	"fill: areaFill: pickup: (in mouseUp:) 
				rect: ellipse: line: polygon: star:"
		ifFalse: ["Don't recognise the command"
			palette setAction: #paint: evt: evt.	"set it to Paint"
			^ self prepareToPaint: evt].! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 6/17/2003 00:05'!
repaint: aForm
	paintingForm ifNil:[
		paintingForm := Form extent: bounds extent depth: 32.
	] ifNotNil:[
		paintingForm fillColor: Color transparent.
	].
	aForm displayOn: paintingForm at: (paintingForm extent - aForm extent // 2).
	self changed.! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 13:35'!
savePainting: aPaintBoxMorph evt: evt
	"Save the image after user issued #keep in aPaintBoxMorph"
	self accept.! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 13:38'!
teapot
	^teapot! !

!TSketchEditorMorph methodsFor: 'accessing' stamp: 'ar 8/20/2002 13:38'!
teapot: aMorph
	teapot := aMorph! !

!TSketchEditorMorph methodsFor: 'painting' stamp: 'ar 8/20/2002 02:11'!
undo: evt
	"revert to a previous state.  "

	| temp poly |

	self flag: #bob.		"what is undo in multihand environment?"
	undoBuffer ifNil: [^ self beep].	"nothing to go back to"
	(poly := self valueOfProperty: #polygon) ifNotNil:
		[poly delete.
		self setProperty: #polygon toValue: nil.
		^ self].
	temp := paintingForm.
	paintingForm := undoBuffer.
	undoBuffer := temp.		"can get back to what you had by undoing again"
	(self paintingFormPen ) setDestForm: paintingForm.
	formCanvas := paintingForm getCanvas.	"used for lines, ovals, etc."
	formCanvas := formCanvas
		copyOrigin: self topLeft negated
		clipRect: (0@0 extent: bounds extent).
	self render: bounds.! !

!TSketchEditorMorph methodsFor: 'palette handling' stamp: 'ar 8/20/2002 02:06'!
undoPainting: aPaintBoxMorph evt: evt
	"Undo the operation after user issued #undo in aPaintBoxMorph"
! !

Morph subclass: #TeaBackingMorph
	instanceVariableNames: 'texture damageRecorder'
	classVariableNames: ''
	poolDictionaries: 'OpenGLConstants'
	category: 'HedgeHacks-Painter'!

!TeaBackingMorph commentStamp: '<historical>' prior: 0!
The sole purpose of a TeaBackingMorph is to act as a backing store for rendering overlays.!

TeaBackingMorph subclass: #TWindowMorph
	instanceVariableNames: 'model frame content windowColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TWindowMorph commentStamp: '<historical>' prior: 0!
TWindowMorph new openInWorld.!

!TWindowMorph methodsFor: 'geometry' stamp: 'ar 9/6/2002 22:29'!
activate
	frame labelColor: Color white.
	content unlock.! !

!TWindowMorph methodsFor: 'geometry' stamp: 'ar 9/8/2002 13:45'!
activateEvent: evt
	owner addMorphFront: self.
! !

!TWindowMorph methodsFor: 'initialize' stamp: 'ar 10/25/2002 23:22'!
closeRequest
	model ifNil:[^self delete].
	model closeRequest.! !

!TWindowMorph methodsFor: 'collapse-expand' stamp: 'ar 9/8/2002 03:46'!
collapse
	self isCollapsed ifTrue:[^self].
	self setProperty: #priorBounds toValue: self bounds.
	content visible: false; delete.
	frame collapse.
	self bounds: (self valueOfProperty: #collapseBounds ifAbsent:[^self]).
! !

!TWindowMorph methodsFor: 'initialize' stamp: 'ar 9/8/2002 03:16'!
collapseRequest
	self isCollapsed ifTrue:[self expand] ifFalse:[self collapse].
! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:32'!
content
	^content! !

!TWindowMorph methodsFor: 'collapse-expand' stamp: 'ar 9/8/2002 04:06'!
expand
	self isCollapsed ifFalse:[^self].
	self setProperty: #collapseBounds toValue: self bounds.
	content visible: true.
	self addMorphBack: content.
	frame expand.
	owner addMorphFront: self.
	self bounds: (self valueOfProperty: #priorBounds ifAbsent:[^self]).
	self removeProperty: #priorBounds.! !

!TWindowMorph methodsFor: 'initialize' stamp: 'ar 9/8/2002 04:06'!
expandRequest
	self isCollapsed ifTrue:[^self expand].
	(self hasProperty: #priorBounds) ifTrue:[
		self bounds: (self valueOfProperty: #priorBounds ifAbsent:[^self]).
		self removeProperty: #priorBounds.
	] ifFalse:[
		self setProperty: #priorBounds toValue: self bounds.
		self bounds: owner bounds.
		owner addMorphFront: self.
	].! !

!TWindowMorph methodsFor: 'geometry' stamp: 'ar 9/6/2002 23:58'!
extent: aPoint
	| textureExtent |
	super extent: aPoint.
	texture ifNil:[^self].
	textureExtent := self extent asLargerPowerOfTwo.
	textureExtent = texture textureForm extent ifFalse:[texture := nil].! !

!TWindowMorph methodsFor: 'events' stamp: 'ar 9/6/2002 22:49'!
followHand: evt frame: aFrame
	| origin |
	evt hand showTemporaryCursor: Cursor normal.
	origin := aFrame valueOfProperty: #dragOrigin ifAbsent:[^self].
	self position: self position + (evt position - origin).
	aFrame setProperty: #dragOrigin toValue: evt position.
	"self addDropShadow.
	self shadowColor: (Color black alpha: 0.3)."! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 21:32'!
frame
	^frame! !

!TWindowMorph methodsFor: 'initialize' stamp: 'ar 9/8/2002 03:20'!
initialize
	super initialize.
	self color: (Color white alpha: 0.9).
	self cornerStyle: #rounded.
	self borderStyle: (BorderStyle raised width: 2).
	self layoutPolicy: TableLayout new.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	frame := TWindowFrameMorph new.
	self on: #mouseDown send: #activateEvent: to: self.
	frame on: #mouseDown send: #startDrag:frame: to: self.
	frame on: #mouseUp send: #stopDrag:frame: to: self.
	frame alphaSlider target: self.
	frame alphaSlider value: self alpha.
	self runScript: #collapseRequest when:{frame collapseButton. #fire}.
	self runScript: #closeRequest when:{frame closeButton. #fire}.
	self runScript: #expandRequest when:{frame expandButton. #fire}.
	self addMorphBack: frame.
	content := (Morph new)
		color: Color transparent;
		layoutPolicy: ProportionalLayout new;
		hResizing: #spaceFill; vResizing: #spaceFill;
		yourself.
	self addMorphBack: content.
	self extent: 400@170.

	self windowColor: Color red.
! !

!TWindowMorph methodsFor: 'testing' stamp: 'ar 9/8/2002 03:16'!
isCollapsed
	^content visible not! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:03'!
label
	^frame label! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:03'!
label: aString
	frame label: aString.! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 10/25/2002 23:25'!
model
	^model! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 10/25/2002 23:25'!
model: anObject
	model := anObject! !

!TWindowMorph methodsFor: 'geometry' stamp: 'ar 9/6/2002 22:40'!
ownerChanged
	"back stop so we can adjust activity state"
	owner ifNotNil:[
		owner firstSubmorph == self 
			ifTrue:[self activate]
			ifFalse:[self passivate].
	].
	^super ownerChanged! !

!TWindowMorph methodsFor: 'geometry' stamp: 'ar 9/6/2002 22:29'!
passivate
	frame labelColor: Color black.
	content lock.! !

!TWindowMorph methodsFor: 'events' stamp: 'ar 9/6/2002 22:27'!
startDrag: evt frame: aFrame
	"Mouse down in window frame"
	owner ifNil:[^self].
	owner firstSubmorph == self ifFalse:[owner addMorphFront: self].
	aFrame setProperty: #dragOrigin toValue: evt position.
	aFrame on: #mouseMove send: #followHand:frame: to: self.
! !

!TWindowMorph methodsFor: 'stepping' stamp: 'ar 9/6/2002 23:39'!
step
	super step.
	self model ifNotNilDo:[:m| m step].! !

!TWindowMorph methodsFor: 'stepping' stamp: 'ar 9/6/2002 23:29'!
stepTime
	^500! !

!TWindowMorph methodsFor: 'events' stamp: 'ar 9/6/2002 22:24'!
stopDrag: evt frame: aFrame
	"Mouse down in window frame"
	aFrame on: #mouseMove send: nil to: nil.
	aFrame removeProperty: #dragOrigin.
	evt hand showTemporaryCursor: nil.
	self removeDropShadow.! !

!TWindowMorph methodsFor: 'stepping' stamp: 'ar 9/6/2002 23:38'!
wantsSteps
	^true! !

!TWindowMorph methodsFor: 'accessing' stamp: 'ar 9/6/2002 22:38'!
windowColor: aColor
	| frameColor |
	windowColor := aColor.
	frame windowColor: windowColor.
	frameColor := (Color white mixed: 0.4 with: windowColor).
	self adoptPaneColor: (Color white mixed: 0.7 with: windowColor).
	self borderStyle baseColor: frameColor.
	content submorphsDo:[:m|
		m borderWidth > 0 ifTrue:[m borderStyle baseColor: frameColor].
	].! !

TeaBackingMorph subclass: #TZoomNavigatorMorph
	instanceVariableNames: 'scroller scrollValue targetHeight focus outer snapshots tools'
	classVariableNames: 'AliceForm BackButtonForm ProjectsForm SnapshotForm ToolFormLibrary ToolsForm'
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TZoomNavigatorMorph commentStamp: '<historical>' prior: 0!
TZoomNavigatorMorph new openInWorld!

!TZoomNavigatorMorph class methodsFor: 'as yet unclassified' stamp: 'ar 1/11/2006 13:47'!
initialize
	"TZoomNavigatorMorph initialize"
	BackButtonForm := self loadIcon2: 'BackButton.gif'.
	ProjectsForm := self loadIcon2: 'Projects.gif'.
	AliceForm := self loadIcon2: 'Alice.gif'.
	SnapshotForm := self loadIcon2: 'Snapshot.gif'.
	ToolsForm := self loadIcon2: 'Tools.gif'.
	ToolFormLibrary := Dictionary new.
	#(TSpaceBrowser TConnectionDialog TPainter) do:[:each|
		ToolFormLibrary at: each put: (self loadIcon2: each,'.gif').
	].! !

!TZoomNavigatorMorph class methodsFor: 'as yet unclassified' stamp: 'ar 1/11/2006 13:46'!
loadIcon2: iconName
	^self loadIcon: iconName defaultExtent: 128@128! !

!TZoomNavigatorMorph class methodsFor: 'as yet unclassified' stamp: 'ar 1/11/2006 13:46'!
loadIcon: iconName defaultExtent: defaultExtent
	| iconFile icon |
	iconFile := [(FileDirectory default directoryNamed: 'icons') readOnlyFileNamed: iconName]
		on: Error do:[:ex| ex return: nil].
	iconFile ifNil:[^Form extent: defaultExtent depth: 8].
	icon := Form fromBinaryStream: iconFile.
	iconFile close.
	^icon! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 6/16/2005 22:43'!
addAliceFile: path image: aForm
	| image |
	image := aForm ifNil:[Form extent: 128@96 depth: 8].
	self drawName: (path copyFrom: (path lastIndexOf: $/) to: path size) on: image.
	self addItem: (TSketchButtonMorph withForm: image)
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #clickOpenAliceFile: argument: path).! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 6/16/2005 22:45'!
addAliceLibrary: path
	| fd fileNames mdlFiles thumbs paths file thumb |
	self addBackButton: #aliceModels.
	fd := FileDirectory on: (FileDirectory pathFromURI: path).
	fileNames := fd fileNames.
	mdlFiles := fileNames select:[:any| any endsWith: '.mdl'].
	thumbs := fileNames select:[:any| any endsWith: '.gif'].
	mdlFiles := mdlFiles asSet.
	thumbs := thumbs asSet.
	paths := fd directoryNames.
	"add paths"
	paths do:[:dir|
		(thumbs includes: dir,'.gif') ifTrue:[
			file := fd readOnlyFileNamed: dir,'.gif'.
			thumb := Form fromBinaryStream: file.
			thumb := thumb asFormOfDepth: 32.
			file close.
		] ifFalse:[thumb := nil].
		self addAlicePath: (path,'/', dir) image: thumb.
	].
	"Now add tea files"
	mdlFiles := mdlFiles asArray sort:[:s1 :s2| (s1 compare: s2) <= 2].
	mdlFiles do:[:fname|
		(thumbs includes: (fname allButLast: 4),'.gif') ifTrue:[
			file := fd readOnlyFileNamed: (fname allButLast: 4),'.gif'.
			thumb := Form fromBinaryStream: file.
			thumb := thumb asFormOfDepth: 32.
			file close.
		] ifFalse:[thumb := nil].
		self addAliceFile: path,'/',fname image: thumb.
	].
! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 6/16/2005 22:33'!
addAlicePath: path image: aForm
	| image |
	aForm ifNil:[image := Form extent: 128@128 depth: 32].
	aForm ifNotNil:[
		image := Form extent: aForm extent depth: 32.
		aForm displayOn: image.
	].
	self drawName: (path copyFrom: (path lastIndexOf: $/) to: path size) on: image.
	self addItem: (TSketchButtonMorph withForm: image)
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #clickOpenAlicePath: argument: path).! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 6/16/2005 22:20'!
addAliceRoot
	self addAlicePath: 'Content/Alice' image: AliceForm.! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 1/22/2006 14:04'!
addAvatarRoot
	self addAlicePath: 'Content/Avatars' image: AliceForm.! !

!TZoomNavigatorMorph methodsFor: 'back button' stamp: 'ar 10/11/2002 23:05'!
addBackButton
	^self addBackButton: nil.! !

!TZoomNavigatorMorph methodsFor: 'back button' stamp: 'ar 10/11/2002 23:07'!
addBackButton: contentType
	| prior button |
	button := TSketchButtonMorph withForm: BackButtonForm.
	button setProperty: #contentType toValue: contentType.
	prior := scroller submorphs.
	scroller removeAllMorphs.
	self addItem: button
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #clickBackTo: argument: prior).! !

!TZoomNavigatorMorph methodsFor: 'initialize' stamp: 'ar 10/11/2002 18:04'!
addItem: sketch
	sketch extent: self smallExtent.
	sketch on: #mouseDown send: #mouseDown:from: to: self.
	sketch on: #mouseUp send: #mouseUp:from: to: self.
	sketch on: #mouseEnter send: #mouseEnter:from: to: self.
	sketch on: #mouseLeave send: #mouseLeave:from: to: self.
	sketch on: #startDrag send: #startDrag:from: to: self.
	scroller addMorphBack: sketch.
	scroller left: self left.! !

!TZoomNavigatorMorph methodsFor: 'initialize' stamp: 'ar 10/8/2002 21:40'!
addItem: sketch dragAction: dragAction clickAction: clickAction
	sketch setProperty: #dragAction toValue: dragAction.
	sketch setProperty: #clickAction toValue: clickAction.
	self addItem: sketch.
! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 10/8/2002 23:15'!
addProject: project

	^self addProject: project 
		clickAction: (MessageSend receiver: self selector: #clickProjectView: argument: project)
		dragAction: (MessageSend receiver: self selector: #dragProjectView: argument: project)
! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 10/8/2002 23:13'!
addProject: project clickAction: clickAction dragAction: dragAction
	| size thumb max scale width height form sketchForm sketch |
	size := self largeExtent.
	thumb := project thumbnail.
	thumb ifNil:[thumb := (Form extent: 128@96 depth: 16) fillWhite].
	max := thumb width max: thumb height.
	scale := size x / max.
	width := (thumb width * scale) truncated.
	height := (thumb height * scale) truncated.
	form := Form extent: width@height depth: 16.
	thumb displayScaledOn: form.
	sketchForm := Form extent: size depth: form depth.
	form displayOn: sketchForm at: sketchForm extent - form extent // (2@1) rule: Form paint.
	self drawName: project name on: sketchForm.
	sketch := SketchMorph withForm: sketchForm.
	self addItem: sketch dragAction: dragAction clickAction: clickAction.! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 3/9/2004 19:30'!
addProjectRoot
	ProjectsForm ifNil:[ProjectsForm := Form extent: 32@32 depth: 32].
	self addProjectRoot: ProjectsForm! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 10/11/2002 21:46'!
addProjectRoot: thumb
	| image |
	image := Form extent: thumb extent depth: 32.
	thumb displayOn: image.
	self drawName: 'Projects' on: image.
	self addItem: (TSketchButtonMorph withForm: image)
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #addProjects).
! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 10/11/2002 23:07'!
addProjects
	self addBackButton: #projects.
	(Project allInstances sort:[:p1 :p2| p1 name <= p2 name]) do:[:prj| self addProject: prj. ].
	targetHeight := nil.! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 10/11/2002 23:10'!
addSnapshotItem: aSketch
	snapshots add: aSketch.
	(scroller submorphs first valueOfProperty: #contentType) == #snapshots ifFalse:[^self].
	self addItem: aSketch
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #clickSnapshot: argument: aSketch).! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 3/9/2004 19:30'!
addSnapshotRoot
	snapshots ifNil:[snapshots := OrderedCollection new].
	SnapshotForm ifNil:[SnapshotForm := Form extent: 32@32 depth: 32].
	self addSnapshotRoot: SnapshotForm.! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 10/11/2002 22:05'!
addSnapshotRoot: thumb
	| image |
	image := Form extent: thumb extent depth: 32.
	thumb displayOn: image.
	self drawName: 'Snapshots' on: image.
	self addItem: (TSketchButtonMorph withForm: image)
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #addSnapshots).
! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 10/11/2002 23:06'!
addSnapshots
	self addBackButton: #snapshots.
	snapshots do:[:item|
		self addItem: item
			dragAction: nil
			clickAction: (MessageSend receiver: self selector: #clickSnapshot: argument: item)].! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 10/12/2002 23:53'!
addTool: toolClass
	| thumb |
	thumb := ToolFormLibrary at: toolClass name ifAbsent:[nil].
	thumb ifNil:[thumb := Form extent: 128@128 depth: 8].
	self addTool: toolClass image: thumb.! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 10/12/2002 23:55'!
addTool: toolClass image: icon
	| sketch image |
	image := Form extent: icon extent depth: 32.
	icon displayOn: image.
	self drawName: toolClass name on: image.
	sketch := TSketchButtonMorph withForm: image.
	sketch setProperty: #clickAction toValue: (MessageSend receiver: self selector: #openTool: argument: toolClass).
	tools add: sketch.! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 10/11/2002 23:07'!
addTools
	self addBackButton: #tools.
	tools do:[:sketch| self addItem: sketch].
	targetHeight := nil.! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 3/9/2004 19:31'!
addToolsRoot
	tools ifNil:[tools := OrderedCollection new].
	ToolsForm ifNil:[ToolsForm := Form extent: 32@32 depth: 32].
	self addToolsRoot: ToolsForm.! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 10/11/2002 22:08'!
addToolsRoot: thumb
	| image |
	image := Form extent: thumb extent depth: 32.
	thumb displayOn: image.
	self drawName: 'Tools' on: image.
	self addItem: (TSketchButtonMorph withForm: image)
		dragAction: nil
		clickAction: (MessageSend receiver: self selector: #addTools).
! !

!TZoomNavigatorMorph methodsFor: 'back button' stamp: 'ar 10/11/2002 19:01'!
clickBackTo: morphs
	scroller removeAllMorphs.
	morphs do:[:each| self addItem: each].
	targetHeight := nil.
	self mouseMove: ActiveHand lastEvent.! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 10/12/2002 01:45'!
clickOpenAliceFile: aPath
	owner clickOpenAliceFile: aPath! !

!TZoomNavigatorMorph methodsFor: 'alice items' stamp: 'ar 10/12/2002 01:45'!
clickOpenAlicePath: aPath
	targetHeight := nil.
	self addAliceLibrary: aPath.
	self mouseMove: ActiveHand lastEvent.! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 1/12/2006 19:00'!
clickProjectView: aProject
	owner ifNotNil:[owner clickProjectView: aProject]! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 2/2/2006 23:22'!
clickSnapshot: item
	owner gotoSnapshot: item.! !

!TZoomNavigatorMorph methodsFor: 'project items' stamp: 'ar 10/8/2002 23:17'!
dragProjectView: aProject
	| project projView |
	project := aProject.
	projView := ProjectViewMorph on: project.
	projView setProperty: #teaDropAction toValue: (MessageSend receiver: nil selector: #makeProjectLink: argument: project).
	ActiveHand attachMorph: projView.! !

!TZoomNavigatorMorph methodsFor: 'initialize' stamp: 'das 9/27/2002 01:51'!
drawName: projectName on: aForm
	| font nameForm box rectForName aCanvas |
	font := TextStyle defaultFont emphasized: 1.
	nameForm := (StringMorph contents: projectName font: font) imageForm.
	nameForm := nameForm scaledToSize: (aForm extent - (4@2) min: nameForm extent).
	box := aForm boundingBox.
	rectForName := box bottomLeft + 
			(box width - nameForm width // 2 @ (nameForm height + 2) negated)
				extent: nameForm extent.
	aCanvas := aForm getCanvas.
	rectForName topLeft eightNeighbors do: [ :pt |
		aCanvas
			stencil: nameForm 
			at: pt
			color: Color white.
	].
	aCanvas
		stencil: nameForm 
		at: rectForName topLeft 
		color: Color black.! !

!TZoomNavigatorMorph methodsFor: 'geometry' stamp: 'ar 9/22/2002 18:42'!
extent: newExtent
	super extent: newExtent.
	scroller position: self position + (self extent - scroller extent // 2).! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 9/22/2002 17:37'!
handleMouseMove: anEvent
	"We need mouseMove even if no button is pressed"
	(anEvent hand hasSubmorphs) ifTrue:[^self].
	anEvent wasHandled: true.
	self mouseMove: anEvent.! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 9/22/2002 17:40'!
handlesMouseOver: event
	^true! !

!TZoomNavigatorMorph methodsFor: 'initialize' stamp: 'ar 10/11/2002 19:00'!
initialize
	super initialize.
	self clipSubmorphs: true.
	self color: Color transparent.
	scroller := Morph new.
	scroller color: Color transparent.
	scroller layoutPolicy: TableLayout new.
	scroller listDirection: #leftToRight.
	scroller hResizing: #shrinkWrap.
	scroller cellPositioning: #bottomCenter.
	scroller wrapCentering: #bottomRight.
	scroller layoutInset: 4@4.
	scroller cellInset: 4@0.
	scroller height: self largeExtent y + 8. "2 x inset"
	self extent: self smallExtent x * 8 + 8 @ scroller height + 16.
	self addMorph: scroller.
! !

!TZoomNavigatorMorph methodsFor: 'accessing' stamp: 'ar 9/22/2002 17:39'!
largeExtent
	^128@128! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 9/22/2002 21:32'!
mouseDown: evt from: projSketch
! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 10/11/2002 19:00'!
mouseEnter: anEvent
	"show me"
	targetHeight := self height.
! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'das 10/1/2002 17:08'!
mouseEnter: evt from: sketch

	focus ifNotNil:[ focus changed. focus framesToDwell: 20].
	focus := sketch.
	focus framesToDwell: 20.
! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 9/22/2002 18:53'!
mouseLeave: anEvent
	"hide me"
	targetHeight := 16.! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'das 10/1/2002 17:08'!
mouseLeave: evt from: sketch

	sketch framesToDwell: 20.
	focus = sketch ifTrue:[focus changed. focus := nil.].
! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 10/11/2002 20:29'!
mouseMove: anEvent
	| xPos rightMorph leftMorph leftFraction rightFraction deltaX leftExtent rightExtent |

	(self containsPoint: anEvent position) ifFalse:[
		scroller submorphsDo:[:m| m extent: self smallExtent].
	].
	"Figure out where we are"
	xPos := anEvent position x.
	"See if we need to scroll"
	scrollValue := nil.
	deltaX := (xPos - bounds left) max: 0.
	deltaX < self largeExtent x ifTrue:[
		scroller left < bounds left ifTrue:[
			scrollValue := self largeExtent x - deltaX // 8.
		].
	].
	deltaX := (bounds right - xPos) max: 0.
	deltaX < self largeExtent x ifTrue:[
		scroller right > bounds right ifTrue:[
			scrollValue := self largeExtent x - deltaX // -8.
		].
	].


	rightMorph := scroller submorphs detect:[:m| m center x > xPos] ifNone:[nil].
	rightMorph ifNil:[
		"We're at the right most point"
		leftMorph := nil.
		leftFraction := 0.0.
		rightMorph := scroller lastSubmorph.
		rightFraction := 1.0.
	] ifNotNil:[
		rightMorph == scroller firstSubmorph ifTrue:[
			"Left-most"
			leftMorph := rightMorph.
			leftFraction := 1.0.
			rightMorph := nil.
			rightFraction := 0.0.
		] ifFalse:[
			"somewhere inbetween"
			leftMorph := scroller submorphs at: (scroller submorphs indexOf: rightMorph)-1.
			deltaX := rightMorph center x - leftMorph center x.
			rightFraction := xPos - leftMorph center x / deltaX asFloat.
			leftFraction := 1.0 - rightFraction.
		].
	].
	leftExtent := self smallExtent + (self largeExtent - self smallExtent * leftFraction) asIntegerPoint.
	rightExtent := self smallExtent + (self largeExtent - self smallExtent * rightFraction) asIntegerPoint.
	scroller submorphsDo:[:m|
		m == leftMorph 
			ifTrue:[m extent: leftExtent]
			ifFalse:[m == rightMorph 
				ifTrue:[m extent: rightExtent]
				ifFalse:[m extent: self smallExtent]]].
! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 10/11/2002 18:51'!
mouseUp: evt from: sketch
	| clickAction |
	clickAction := sketch valueOfProperty: #clickAction.
	clickAction ifNil:[^self].
	self mouseLeave: evt.
	clickAction value.
! !

!TZoomNavigatorMorph methodsFor: 'tools' stamp: 'ar 10/11/2002 22:46'!
openTool: toolClass
	owner openTool: toolClass.! !

!TZoomNavigatorMorph methodsFor: 'geometry' stamp: 'ar 10/12/2002 18:32'!
ownerChanged
	self left > owner left ifTrue:[self left: owner left].
	self top > owner bottom ifTrue:[self top: owner bottom - 16].
	self bottom < owner bottom ifTrue:[self top: owner bottom - 16].
	self extent x = owner extent x ifFalse:[super extent: owner extent x @ scroller height + 32].! !

!TZoomNavigatorMorph methodsFor: 'initialize' stamp: 'ar 10/11/2002 16:48'!
removeAll
	scroller removeAllMorphs! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 5/21/2003 14:25'!
resetSnapshots
	snapshots := OrderedCollection new.
! !

!TZoomNavigatorMorph methodsFor: 'accessing' stamp: 'ar 9/22/2002 17:39'!
smallExtent
	^64@64! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 2/2/2006 23:26'!
snapshots
	^snapshots! !

!TZoomNavigatorMorph methodsFor: 'snapshot items' stamp: 'ar 2/2/2006 23:22'!
snapshots: aCollection
	snapshots := aCollection! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 10/8/2002 21:46'!
startDrag: evt from: sketch
	| dragAction |
	dragAction := sketch valueOfProperty: #dragAction.
	dragAction ifNil:[^self].
	dragAction value.
	self mouseLeave: evt.! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 2/2/2006 23:37'!
step
	"Step the receiver"
	| height |
	focus ifNotNil:[
		focus framesToDwell: (focus framesToDwell-1).
		focus changed.
	].
	scrollValue ifNotNil:[
		scroller left: scroller left + scrollValue.
		scrollValue < 0 ifTrue:["scroll left"
			scroller right <= bounds right ifTrue:[
				scroller right: bounds right.
				scrollValue := nil].
		] ifFalse:[
			scroller left >= bounds left ifTrue:[
				scroller left: bounds left.
				scrollValue := nil].
		].
		self mouseMove: ActiveHand lastEvent.
	].
	targetHeight ifNotNil:[
		height := owner bottom - self top.
		height = targetHeight ifTrue:[^targetHeight := nil].
		height < targetHeight
			ifTrue:[height := height + 16 min: targetHeight]
			ifFalse:[height := height - 16 max: targetHeight].
		self top: owner bottom - height.
	].! !

!TZoomNavigatorMorph methodsFor: 'event handling' stamp: 'ar 9/22/2002 18:18'!
stepTime
	^50! !

!TeaBackingMorph methodsFor: 'rendering' stamp: 'bgf 5/16/2006 22:00'!
glRenderOn: ogl
	"Update what's dirty and blast out the texture"
	| tw th box mat fakeSHA |
	texture ifNil:[
		fakeSHA := TFormManager generateSHA: (TObjectID for: 'TeaBackingMorph SHA').
		texture := TTexture 
			new initializeWithForm: (Form extent: self extent x asLargerPowerOfTwo @ self extent y asLargerPowerOfTwo depth: 32)
			mipmap: false 
			shrinkFit: true.
		mat := TMaterial new.
		mat ambientColor: #(1.0 1.0 1.0 0.9) asFloatArray.
		mat diffuseColor: #(1.0 1.0 1.0 0.9) asFloatArray.
		mat emissiveColor: #(1.0 1.0 1.0 1.0) asFloatArray.
		texture material: mat.
		texture tform fakeSHA: fakeSHA.
	].
	damageRecorder updateIsNeeded ifTrue:[self repairDamage].
	texture enable: ogl.
	tw := bounds width asFloat / texture targetForm width asFloat.
	th := bounds height asFloat / texture targetForm height asFloat.
	box := bounds translateBy: owner bounds origin negated.
	ogl glDisable: GLLighting.
	ogl glColor4f: 1.0 with: 1.0 with: 1.0 with: 1.0.
	ogl glBegin: GLQuads;
		glTexCoord2f: 0.0 with: 0.0;
		glVertex2f: box left with: box top;

		glTexCoord2f: tw with: 0.0;
		glVertex2f: box right with: box top;

		glTexCoord2f: tw with: th;
		glVertex2f: box right with: box bottom;

		glTexCoord2f: 0.0 with: th;
		glVertex2f: box left with: box bottom;

		glEnd.
	texture disable: ogl.
! !

!TeaBackingMorph methodsFor: 'initialize' stamp: 'ar 8/19/2002 19:51'!
initialize
	super initialize.
	damageRecorder := DamageRecorder new.
	damageRecorder doFullRepaint.! !

!TeaBackingMorph methodsFor: 'rendering' stamp: 'ar 9/8/2002 03:26'!
invalidRect: aRectangle from: aMorph
	| damageRect |
	texture ifNil:[^super invalidRect: aRectangle from: aMorph].
	damageRect := aRectangle.
	aMorph == self ifFalse:[
		"Clip to receiver's clipping bounds if the damage came from a child"
		self clipSubmorphs 
			ifTrue:[damageRect := aRectangle intersect: self clippingBounds]].
	damageRecorder recordInvalidRect: (damageRect translateBy: bounds origin negated).! !

!TeaBackingMorph methodsFor: 'geometry' stamp: 'ar 9/6/2002 22:53'!
position: aPoint
	| delta |
	texture ifNil:[^super position: aPoint].
	delta := aPoint - bounds topLeft.
	(delta x = 0 and: [delta y = 0]) ifTrue: [^ self].  "Null change"
	self privateFullMoveBy: delta.
	owner ifNotNil:[owner layoutChanged].
! !

!TeaBackingMorph methodsFor: 'rendering' stamp: 'ar 2/1/2006 14:54'!
repairDamage
	| rectList fc bb |
	rectList := damageRecorder invalidRectsFullBounds: (0@0 extent: self extent).
	damageRecorder reset.
	bb := BitBlt toForm: texture targetForm.
	bb combinationRule: 40 "fixAlpha:with:".
	bb halftoneForm: (Bitmap with: 16rFF000000).
	fc := (FormCanvas on: texture targetForm).
	rectList do:[:r|
		fc clipBy: r during:[:cc| 
			cc fillColor: Color transparent.
			cc translateBy: bounds origin negated during:[:cc2 | self fullDrawOn: cc2]].
		bb destRect: r.
		bb copyBits.
		texture invalidate: r.
	].! !

SubdivisionQuadEdge subclass: #T3DQuadEdge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DQuadEdge methodsFor: 'private' stamp: 'ar 1/11/2006 14:33'!
edgeClass
	^T3DHalfEdge! !

Subdivision subclass: #T3DSubdivision
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DSubdivision class methodsFor: 'instance creation' stamp: 'ar 3/8/2006 19:33'!
make3DMeshFrom: aForm style: extrusionStyle
	"
		self make3DMeshFrom: (Form fromFileNamed: 'Sketches\Goat.gif') style: #round
	"
	| list pts subdivision mask |
	mask := Form extent: aForm extent*2 depth: 1.
	(WarpBlt toForm: mask)
		sourceForm: aForm destRect: mask boundingBox;
		combinationRule: Form over;
		cellSize: 1;
		colorMap: (Color maskingMap: aForm depth);
		warpBits.

	subdivision := T3DSubdivision withSize: (mask boundingBox).
	list := mask traceOutlines.
	list := list collect:[:poly|
		poly collect:[:loop|
			pts := StrokeSimplifier new.
			loop do:[:pt| pts add: pt].
			pts closeStroke.
			pts := pts finalStroke.
			pts := StrokeSimplifier smoothen: pts length: 10.
			pts := LineIntersections regularize: pts.
			pts do:[:pt| subdivision insertPoint: pt].
			pts]].
	list := list collect:[:poly|
		poly collect:[:loop| subdivision constraintOutline: loop].
	].

	list do:[:poly|
		poly keysAndValuesDo:[:index :loop|
			subdivision markExteriorEdges: (index = 1) in: loop.
		].
	].
	extrusionStyle == #round
		ifTrue:[^subdivision build3DObject: false].
	extrusionStyle == #flat
		ifTrue:[^subdivision surface].
	^subdivision surface! !

!T3DSubdivision methodsFor: 'private' stamp: 'daf 11/7/2004 16:10'!
build3DObject
	"Return the full triangulation of the receiver"
	^self build3DObject: Preferences twoSidedPopoutTextures ! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 3/8/2006 19:38'!
build3DObject: twoSided
	"Return the full triangulation of the receiver"
	| firstPoly poly faces mesh half vtx nrm texScale |
	self markExteriorEdges.
	firstPoly := self triangulate.
	faces := WriteStream on: #().
	poly := firstPoly.
	[poly == nil] whileFalse:[
		faces nextPutAll: (poly elevateSpine: 2).
		poly := poly next].
	faces := faces contents.
	faces isEmpty ifTrue:[^nil].
	half := T3DSimpleMesh withAll: faces contents.
	texScale := 1.0 / (area width max: area height).
	twoSided 
		ifTrue:[texScale := texScale @ (0.5 * texScale)]
		ifFalse:[texScale := (1.0 / area width) @ (1.0 / area height)].
	half faces do:[:face| face vertices do:[:v|
		v texCoord: (v position x @ v position y) - area origin * texScale]].
	half := half asIndexedMesh.
	half vertexNormals: (half computeVertexNormals collect:[:v| v negated]).
	vtx := half vertices.
	nrm := half vertexNormals.
	1 to: vtx size do:[:i|
		(vtx at: i) z = 0.0 ifTrue:[nrm at: i put: ((nrm at: i) z: 0.0)]].
	mesh := self mirror: half twoSided: twoSided.
	^mesh! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 1/11/2006 14:34'!
createPolygons
	| poly firstPoly |
	firstPoly := nil.
	self innerTriangleEdgesDo:[:e1 :e2 :e3|
		poly := T3DTriangle new.
		poly e1: e1 e2: e2 e3: e3.
		poly recomputeType.
		poly next: firstPoly.
		firstPoly := poly.
	].
	^firstPoly! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 3/8/2006 20:09'!
mirror: half twoSided: twoSided
	"Mirror one half-mesh and return a full mesh"
	| vtx nrm tex faces faceOffset vtxOffset mesh f |
	vtx := Vector3Array new: half vertices size * 2.
	nrm := Vector3Array new: half vertexNormals size * 2.
	tex := Vector2Array new: half texCoords size * 2.
	faces := Array new: (half faces size * 2).
	"Fill in first half"
	vtx 
		privateReplaceFrom: 1 
		to: half vertices basicSize 
		with: half vertices 
		startingAt: 1.
	nrm 
		privateReplaceFrom: 1 
		to: half vertexNormals basicSize 
		with: half vertexNormals 
		startingAt: 1.
	tex 
		privateReplaceFrom: 1 
		to: half texCoords basicSize 
		with: half texCoords 
		startingAt: 1.
	faces 
		replaceFrom: 1 
		to: half faces size 
		with: half faces 
		startingAt: 1.

	"fill in second half"
	half flipZ.
	twoSided ifTrue:[
		half texCoords do:[:tx| tx v: (1.0 - tx v)].
	].
	vtx 
		privateReplaceFrom: half vertices basicSize + 1 
		to: vtx basicSize 
		with: half vertices 
		startingAt: 1.
	nrm 
		privateReplaceFrom: half vertexNormals basicSize + 1 
		to: nrm basicSize 
		with: half vertexNormals 
		startingAt: 1.
	tex 
		privateReplaceFrom: half texCoords basicSize + 1 
		to: tex basicSize 
		with: half texCoords 
		startingAt: 1.

	faceOffset := half faces size.
	vtxOffset := half vertices size.
	1 to: half faces size do:[:i|
		f := half faces at: i.
		faces at: i+faceOffset put: (Array
			with: (f at: 3) + vtxOffset
			with: (f at: 2) + vtxOffset
			with: (f at: 1) + vtxOffset
		).
	].
	mesh := T3DTriangleMesh new.
	mesh vertices: vtx.
	mesh vertexNormals: nrm.
	mesh faces: faces.
	mesh texCoords: tex.
	^mesh! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 1/11/2006 14:33'!
quadEdgeClass
	^T3DQuadEdge! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 3/8/2006 19:38'!
surface
	"Return the mesh representing the surface of the subdivision"
	| firstPoly poly faces half texScale |
	self markExteriorEdges.
	firstPoly := self triangulate.
	faces := WriteStream on: #().
	poly := firstPoly.
	[poly == nil] whileFalse:[
		faces nextPut: (T3DSimpleFace 
			with: (T3DSimpleVertex new position: poly v1@0) 
			with: (T3DSimpleVertex new position: poly v2@0)
			with: (T3DSimpleVertex new position: poly v3@0)).
		poly := poly next].
	faces := faces contents.
	faces isEmpty ifTrue:[^nil].
	half := T3DSimpleMesh withAll: faces contents.
	texScale := 1.0 / (area width max: area height).
	texScale := (1.0 / area width) @ (1.0 / area height).
	half do:[:face| face do:[:v|
		v texCoord: (v position x @ v position y) - area origin * texScale]].
	half := half asIndexedMesh.
	half vertexNormals: (half computeVertexNormals collect:[:v| v negated]).
	^half! !

!T3DSubdivision methodsFor: 'private' stamp: 'ar 5/22/2001 13:29'!
triangulate
	"Return the full triangulation of the receiver"
	| firstPoly poly |
	"Create the triangle list"
	firstPoly := self createPolygons.
	poly := firstPoly.
	[poly == nil] whileFalse:[
		poly buildTerminalFan.
		poly := poly next].
	^firstPoly! !

!StringMorph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/6/2006 22:44'!
labelColor
	^self color! !

!StringMorph methodsFor: '*HedgeHacks-Painter' stamp: 'ar 1/6/2006 22:44'!
labelColor: aColor
	self color: aColor.! !

SketchMorph subclass: #TSketchButtonMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TSketchButtonMorph methodsFor: 'as yet unclassified' stamp: 'ar 10/8/2002 23:04'!
containsPoint: aPoint
	^self bounds containsPoint: aPoint! !

SubdivisionHalfEdge subclass: #T3DHalfEdge
	instanceVariableNames: 'leftFace rightFace fanVertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter-T3D'!

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 22:57'!
elevationHeight
	| height center |
	height := self length * 0.5.
	fanVertices ifNotNil:[
		center := self center.
		fanVertices do:[:v| height := height + (v dist: center)].
		height := height / (fanVertices size + 1) asFloat].
	^height! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 20:46'!
fanVertices
	^fanVertices! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 20:46'!
fanVertices: pointList
	fanVertices := pointList! !

!T3DHalfEdge methodsFor: 'testing' stamp: 'ar 5/19/2001 19:26'!
hasAllInsideCircle: aCollection 
	"Tests if all Points in aCollection are inside the circle which has this  
	edge as a diameter"
	| start end |
	start := self origin.
	end := self destination.
	aCollection do: [:aPoint | (start - aPoint dotProduct: end - aPoint)
			> 0 ifTrue: [^ false]].
	^ true! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:37'!
leftFace
	^leftFace! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:09'!
leftFace: aFace
	self assert:[leftFace == nil].
	leftFace := aFace! !

!T3DHalfEdge methodsFor: 'private' stamp: 'ar 1/11/2006 14:33'!
quadEdgeClass
	^T3DQuadEdge! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:37'!
rightFace
	^rightFace! !

!T3DHalfEdge methodsFor: 'accessing' stamp: 'ar 5/19/2001 17:09'!
rightFace: aFace
	self assert:[rightFace == nil].
	rightFace := aFace! !

AlignmentMorphBob1 subclass: #TLoginMorph
	instanceVariableNames: 'theName theNameMorph thePasswordMorph actionBlock cancelBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HedgeHacks-Painter'!

!TLoginMorph commentStamp: '<historical>' prior: 0!
TLoginMorph new openInWorld!

!TLoginMorph methodsFor: 'building' stamp: 'ar 12/26/2002 16:06'!
buttonColor

	^color darker! !

!TLoginMorph methodsFor: 'building' stamp: 'ar 12/26/2002 16:22'!
buttonNamed: aString action: aSymbol color: aColor help: helpString

	| f col |
	f := SimpleButtonMorph new
		target: self;
		label: aString font: self myFont;
		color: aColor;
		actionSelector: aSymbol;
		setBalloonText: helpString.
	f width: 80.
	f cornerStyle: #square.
	f color: (Color r: 0.95 g: 0.95 b: 1.0).
	f borderStyle: (BorderStyle raised width: 2).
	col := (self inAColumn: {f}) hResizing: #spaceFill.
	^col! !

!TLoginMorph methodsFor: 'building' stamp: 'ar 12/26/2002 16:06'!
cancelButton

	^self
		buttonNamed: 'Cancel' 
		action: #doCancel 
		color: self buttonColor 
		help: 'Cancel this login operation.'! !

!TLoginMorph methodsFor: 'actions' stamp: 'ar 12/26/2002 16:05'!
doCancel

	self delete.
	cancelBlock ifNotNil:[cancelBlock value].! !

!TLoginMorph methodsFor: 'actions' stamp: 'ar 12/26/2002 16:21'!
doOK

	| user password |
	user := theNameMorph contents string.
	password := thePasswordMorph contents string.
	(actionBlock value: user value: password) ifTrue:[self delete].! !

!TLoginMorph methodsFor: 'initialize' stamp: 'ar 12/26/2002 16:32'!
initialize
	| fs |
	super initialize.
	self beSticky.
	fs := GradientFillStyle ramp: {0.0 -> (Color r: 0.8 g: 0.8 b: 1.0). 1.0 -> (Color white) }.
	self vResizing: #shrinkWrap.
	self hResizing: #shrinkWrap.
	color := Color paleYellow.
	borderWidth := 4.
	borderColor := #complexRaised.
	self layoutInset: (4@6).
	self rebuild.
	fs origin: bounds origin.
	fs direction: 0@self fullBounds height.
	self fillStyle: fs.! !

!TLoginMorph methodsFor: 'building' stamp: 'ar 12/26/2002 16:06'!
myFont

	^(TextStyle named: #ComicBold) fontOfSize: 16! !

!TLoginMorph methodsFor: 'initialize' stamp: 'ar 12/26/2002 16:05'!
name: aString actionBlock: aBlock cancelBlock: altBlock

	theName := aString.
	actionBlock := aBlock.
	cancelBlock := altBlock.
	theNameMorph contentsWrapped: theName.
	theNameMorph editor selectAll.! !

!TLoginMorph methodsFor: 'building' stamp: 'ar 12/26/2002 16:06'!
okButton

	^self
		buttonNamed: 'OK' 
		action: #doOK 
		color: self buttonColor 
		help: 'Login into Squeak'! !

!TLoginMorph methodsFor: 'initialize' stamp: 'ar 12/26/2002 16:05'!
openInWorld: aWorld
	super openInWorld: aWorld.
	aWorld primaryHand newKeyboardFocus: theNameMorph.! !

!TLoginMorph methodsFor: 'initialize' stamp: 'ar 12/26/2002 16:20'!
rebuild

	self removeAllMorphs.
	"self addARow: { (StringMorph contents:'Please enter your credentials') lock }."
	self addARow: {
		(StringMorph contents: 'User name' font: self myFont) lock.
	}.
	(self addARow: {
		(theNameMorph := TextMorph new
			beAllFont: self myFont;
			crAction: (MessageSend receiver: self selector: #doOK);
			extent: 236@20;
			contentsWrapped: '<user name>';
			setBalloonText: 'Enter your name'
			).
	}) color: Color white; borderColor: Color black; borderWidth: 1.

	self addARow: {
		(StringMorph contents: 'Password:' font: self myFont) lock.
	}.
	(self addARow: {
		(thePasswordMorph := TextMorph new
			beAllFont: (StrikeFont passwordFontSize: 16);
			crAction: (MessageSend receiver: self selector: #doOK);
			extent: 236@20;
			contentsWrapped: '<password>';
			setBalloonText: 'Enter your password'
			).
	}) color: Color white; borderColor: Color black; borderWidth: 1.

	self addARow: {
		self okButton.
		self cancelButton.
	}.
	"self addARow: { (StringMorph contents:'') lock }."
! !
TWindowFrameMorph initialize!
TZoomNavigatorMorph initialize!
